/*
Copyright (C) 2018-2019 de4dot@gmail.com

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

use super::super::super::iced_constants::IcedConstants;
use super::super::fmt_consts::*;
use super::FormatterString;
#[cfg(not(feature = "std"))]
use alloc::vec::Vec;

pub(super) struct Info {
	pub(super) keyword: &'static FormatterString,
	pub(super) bcst_to: &'static FormatterString,
}

#[allow(non_camel_case_types)]
#[derive(Copy, Clone)]
enum MemoryKeywords {
	None,
	byte,
	dword,
	far,
	fpuenv14,
	fpuenv28,
	fpustate108,
	fpustate94,
	oword,
	qword,
	tword,
	word,
	yword,
	zword,
}

#[allow(non_camel_case_types)]
enum BroadcastToKind {
	b1to2,
	b1to4,
	b1to8,
	b1to16,
}

// GENERATOR-BEGIN: BcstTo
// ⚠️This was generated by GENERATOR!🦹‍♂️
#[cfg_attr(feature = "cargo-fmt", rustfmt::skip)]
static BCST_TO_DATA: [BroadcastToKind; 36] = [
	BroadcastToKind::b1to2,
	BroadcastToKind::b1to2,
	BroadcastToKind::b1to2,
	BroadcastToKind::b1to4,
	BroadcastToKind::b1to4,
	BroadcastToKind::b1to2,
	BroadcastToKind::b1to2,
	BroadcastToKind::b1to2,
	BroadcastToKind::b1to4,
	BroadcastToKind::b1to2,
	BroadcastToKind::b1to8,
	BroadcastToKind::b1to8,
	BroadcastToKind::b1to4,
	BroadcastToKind::b1to4,
	BroadcastToKind::b1to4,
	BroadcastToKind::b1to8,
	BroadcastToKind::b1to4,
	BroadcastToKind::b1to16,
	BroadcastToKind::b1to16,
	BroadcastToKind::b1to8,
	BroadcastToKind::b1to8,
	BroadcastToKind::b1to8,
	BroadcastToKind::b1to16,
	BroadcastToKind::b1to8,
	BroadcastToKind::b1to4,
	BroadcastToKind::b1to8,
	BroadcastToKind::b1to16,
	BroadcastToKind::b1to2,
	BroadcastToKind::b1to4,
	BroadcastToKind::b1to8,
	BroadcastToKind::b1to2,
	BroadcastToKind::b1to4,
	BroadcastToKind::b1to8,
	BroadcastToKind::b1to4,
	BroadcastToKind::b1to8,
	BroadcastToKind::b1to16,
];
// GENERATOR-END: BcstTo

// GENERATOR-BEGIN: MemorySizes
// ⚠️This was generated by GENERATOR!🦹‍♂️
#[cfg_attr(feature = "cargo-fmt", rustfmt::skip)]
static MEM_SIZE_TBL_DATA: [MemoryKeywords; 136] = [
	MemoryKeywords::None,
	MemoryKeywords::byte,
	MemoryKeywords::word,
	MemoryKeywords::dword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::oword,
	MemoryKeywords::yword,
	MemoryKeywords::zword,
	MemoryKeywords::byte,
	MemoryKeywords::word,
	MemoryKeywords::dword,
	MemoryKeywords::qword,
	MemoryKeywords::oword,
	MemoryKeywords::yword,
	MemoryKeywords::zword,
	MemoryKeywords::far,
	MemoryKeywords::far,
	MemoryKeywords::far,
	MemoryKeywords::word,
	MemoryKeywords::dword,
	MemoryKeywords::qword,
	MemoryKeywords::None,
	MemoryKeywords::None,
	MemoryKeywords::qword,
	MemoryKeywords::oword,
	MemoryKeywords::None,
	MemoryKeywords::None,
	MemoryKeywords::word,
	MemoryKeywords::dword,
	MemoryKeywords::qword,
	MemoryKeywords::tword,
	MemoryKeywords::oword,
	MemoryKeywords::word,
	MemoryKeywords::fpuenv14,
	MemoryKeywords::fpuenv28,
	MemoryKeywords::fpustate94,
	MemoryKeywords::fpustate108,
	MemoryKeywords::None,
	MemoryKeywords::None,
	MemoryKeywords::None,
	MemoryKeywords::None,
	MemoryKeywords::tword,
	MemoryKeywords::word,
	MemoryKeywords::word,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::oword,
	MemoryKeywords::oword,
	MemoryKeywords::oword,
	MemoryKeywords::oword,
	MemoryKeywords::oword,
	MemoryKeywords::oword,
	MemoryKeywords::oword,
	MemoryKeywords::oword,
	MemoryKeywords::oword,
	MemoryKeywords::oword,
	MemoryKeywords::oword,
	MemoryKeywords::oword,
	MemoryKeywords::oword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::yword,
	MemoryKeywords::zword,
	MemoryKeywords::zword,
	MemoryKeywords::zword,
	MemoryKeywords::zword,
	MemoryKeywords::zword,
	MemoryKeywords::zword,
	MemoryKeywords::zword,
	MemoryKeywords::zword,
	MemoryKeywords::zword,
	MemoryKeywords::zword,
	MemoryKeywords::zword,
	MemoryKeywords::zword,
	MemoryKeywords::zword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::dword,
	MemoryKeywords::qword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::dword,
	MemoryKeywords::qword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::dword,
	MemoryKeywords::qword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::qword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
	MemoryKeywords::dword,
];
// GENERATOR-END: MemorySizes

lazy_static! {
	pub(super) static ref MEM_SIZE_TBL: Vec<Info> = {
		let mut v = Vec::with_capacity(IcedConstants::NUMBER_OF_MEMORY_SIZES);
		let c = &*FORMATTER_CONSTANTS;
		for (i, &mem_keywords) in MEM_SIZE_TBL_DATA.iter().enumerate() {
			let keyword = match mem_keywords {
				MemoryKeywords::None => &c.empty,
				MemoryKeywords::byte => &c.byte,
				MemoryKeywords::dword => &c.dword,
				MemoryKeywords::far => &c.far,
				MemoryKeywords::fpuenv14 => &c.fpuenv14,
				MemoryKeywords::fpuenv28 => &c.fpuenv28,
				MemoryKeywords::fpustate108 => &c.fpustate108,
				MemoryKeywords::fpustate94 => &c.fpustate94,
				MemoryKeywords::oword => &c.oword,
				MemoryKeywords::qword => &c.qword,
				MemoryKeywords::tword => &c.tword,
				MemoryKeywords::word => &c.word,
				MemoryKeywords::yword => &c.yword,
				MemoryKeywords::zword => &c.zword,
			};
			let bcst_to = if i < IcedConstants::FIRST_BROADCAST_MEMORY_SIZE as usize {
				&c.empty
			} else {
				match BCST_TO_DATA[i - IcedConstants::FIRST_BROADCAST_MEMORY_SIZE as usize] {
					BroadcastToKind::b1to2 => &c.b1to2,
					BroadcastToKind::b1to4 => &c.b1to4,
					BroadcastToKind::b1to8 => &c.b1to8,
					BroadcastToKind::b1to16 => &c.b1to16,
				}
			};

			v.push(Info { keyword, bcst_to });
		}
		v
	};
}
