/*
Copyright (C) 2018-2019 de4dot@gmail.com

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

using System;
using System.Linq;
using Generator.Enums;

namespace Generator.Decoder {
	static class InstructionMemorySizesTable {
		public static readonly (EnumValue codeEnum, EnumValue mem, EnumValue bcst)[] Table = CreateTable();

		static (EnumValue codeEnum, EnumValue mem, EnumValue bcst)[] CreateTable() {
			var memSize = MemorySizeEnum.Instance;
			var code = CodeEnum.Instance;
			var result = new (EnumValue codeEnum, EnumValue mem, EnumValue bcst)[] {
				(code[nameof(Code.INVALID)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.DeclareByte)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.DeclareWord)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.DeclareDword)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.DeclareQword)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_r8_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_AL_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_AX_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_EAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_RAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushw_ES)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushd_ES)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popw_ES)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popd_ES)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_r8_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_AL_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_AX_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_EAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_RAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushw_CS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushd_CS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popw_CS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_r8_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_AL_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_AX_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_EAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_RAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushw_SS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushd_SS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popw_SS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popd_SS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_r8_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_AL_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_AX_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_EAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_RAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushw_DS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushd_DS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popw_DS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popd_DS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_r8_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_AL_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_AX_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_EAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_RAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Daa)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_r8_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_AL_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_AX_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_EAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_RAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Das)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_r8_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_AL_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_AX_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_EAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_RAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Aaa)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_r8_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_AL_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_AX_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_EAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_RAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Aas)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Inc_r16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Inc_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Dec_r16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Dec_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Push_r16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Push_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Push_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pop_r16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pop_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pop_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushaw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushad)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popaw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popad)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bound_r16_m1616)], memSize[nameof(MemorySize.Bound16_WordWord)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bound_r32_m3232)], memSize[nameof(MemorySize.Bound32_DwordDword)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Arpl_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Arpl_r32m16_r32)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsxd_r16_rm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsxd_r32_rm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsxd_r64_rm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Push_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushd_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushq_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_r16_rm16_imm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_r32_rm32_imm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_r64_rm64_imm32)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushw_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushd_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushq_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_r16_rm16_imm8)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_r32_rm32_imm8)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_r64_rm64_imm8)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Insb_m8_DX)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Insw_m16_DX)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Insd_m32_DX)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Outsb_DX_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Outsw_DX_m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Outsd_DX_m32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jo_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jo_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jo_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jno_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jno_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jno_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jb_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jb_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jb_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jae_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jae_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jae_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Je_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Je_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Je_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jne_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jne_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jne_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jbe_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jbe_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jbe_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ja_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ja_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ja_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Js_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Js_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Js_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jns_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jns_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jns_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jp_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jp_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jp_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jnp_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jnp_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jnp_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jl_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jl_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jl_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jge_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jge_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jge_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jle_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jle_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jle_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jg_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jg_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jg_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_rm16_imm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_rm64_imm32)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_rm16_imm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_rm64_imm32)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_rm16_imm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_rm64_imm32)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_rm16_imm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_rm64_imm32)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_rm16_imm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_rm64_imm32)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_rm16_imm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_rm64_imm32)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_rm16_imm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_rm64_imm32)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_rm16_imm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_rm64_imm32)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_rm8_imm8_82)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_rm8_imm8_82)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_rm8_imm8_82)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_rm8_imm8_82)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_rm8_imm8_82)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_rm8_imm8_82)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_rm8_imm8_82)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_rm8_imm8_82)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Add_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Or_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adc_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sbb_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.And_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sub_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xor_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmp_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xchg_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xchg_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xchg_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xchg_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r8_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_rm16_Sreg)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r32m16_Sreg)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r64m16_Sreg)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lea_r16_m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lea_r32_m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lea_r64_m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_Sreg_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_Sreg_r32m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_Sreg_r64m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pop_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pop_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pop_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Nopw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Nopd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Nopq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xchg_r16_AX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xchg_r32_EAX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xchg_r64_RAX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pause)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cbw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cwde)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cdqe)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cwd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cdq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cqo)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Call_ptr1616)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Call_ptr1632)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wait)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushfw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushfd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushfq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popfw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popfd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popfq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sahf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lahf)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_AL_moffs8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_AX_moffs16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_EAX_moffs32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_RAX_moffs64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_moffs8_AL)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_moffs16_AX)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_moffs32_EAX)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_moffs64_RAX)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsb_m8_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsw_m16_m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsd_m32_m32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsq_m64_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpsb_m8_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpsw_m16_m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpsd_m32_m32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpsq_m64_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_AL_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_AX_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_EAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_RAX_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Stosb_m8_AL)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Stosw_m16_AX)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Stosd_m32_EAX)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Stosq_m64_RAX)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lodsb_AL_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lodsw_AX_m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lodsd_EAX_m32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lodsq_RAX_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Scasb_AL_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Scasw_AX_m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Scasd_EAX_m32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Scasq_RAX_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r8_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r16_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r32_imm32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r64_imm64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rol_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ror_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcl_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcr_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shl_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shr_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sal_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sar_rm8_imm8)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rol_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rol_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rol_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ror_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ror_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ror_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcl_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcl_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcl_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcr_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcr_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcr_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shl_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shl_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shl_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shr_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shr_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shr_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sal_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sal_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sal_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sar_rm16_imm8)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sar_rm32_imm8)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sar_rm64_imm8)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Retnw_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Retnd_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Retnq_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Retnw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Retnd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Retnq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Les_r16_m1616)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Les_r32_m1632)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lds_r16_m1616)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lds_r32_m1632)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xabort_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_rm16_imm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_rm64_imm32)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xbegin_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xbegin_rel32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Enterw_imm16_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Enterd_imm16_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Enterq_imm16_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Leavew)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Leaved)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Leaveq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Retfw_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Retfd_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Retfq_imm16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Retfw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Retfd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Retfq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Int3)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Int_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Into)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Iretw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Iretd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Iretq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rol_rm8_1)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ror_rm8_1)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcl_rm8_1)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcr_rm8_1)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shl_rm8_1)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shr_rm8_1)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sal_rm8_1)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sar_rm8_1)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rol_rm16_1)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rol_rm32_1)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rol_rm64_1)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ror_rm16_1)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ror_rm32_1)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ror_rm64_1)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcl_rm16_1)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcl_rm32_1)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcl_rm64_1)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcr_rm16_1)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcr_rm32_1)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcr_rm64_1)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shl_rm16_1)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shl_rm32_1)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shl_rm64_1)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shr_rm16_1)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shr_rm32_1)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shr_rm64_1)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sal_rm16_1)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sal_rm32_1)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sal_rm64_1)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sar_rm16_1)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sar_rm32_1)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sar_rm64_1)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rol_rm8_CL)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ror_rm8_CL)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcl_rm8_CL)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcr_rm8_CL)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shl_rm8_CL)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shr_rm8_CL)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sal_rm8_CL)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sar_rm8_CL)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rol_rm16_CL)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rol_rm32_CL)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rol_rm64_CL)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ror_rm16_CL)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ror_rm32_CL)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ror_rm64_CL)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcl_rm16_CL)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcl_rm32_CL)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcl_rm64_CL)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcr_rm16_CL)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcr_rm32_CL)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcr_rm64_CL)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shl_rm16_CL)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shl_rm32_CL)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shl_rm64_CL)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shr_rm16_CL)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shr_rm32_CL)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shr_rm64_CL)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sal_rm16_CL)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sal_rm32_CL)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sal_rm64_CL)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sar_rm16_CL)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sar_rm32_CL)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sar_rm64_CL)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Aam_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Aad_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Salc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xlat_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fadd_m32fp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fmul_m32fp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcom_m32fp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcomp_m32fp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsub_m32fp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsubr_m32fp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fdiv_m32fp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fdivr_m32fp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fadd_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fmul_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcom_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcomp_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsub_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsubr_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fdiv_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fdivr_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fld_m32fp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fst_m32fp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstp_m32fp)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fldenv_m14byte)], memSize[nameof(MemorySize.FpuEnv14)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fldenv_m28byte)], memSize[nameof(MemorySize.FpuEnv28)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fldcw_m2byte)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fnstenv_m14byte)], memSize[nameof(MemorySize.FpuEnv14)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstenv_m14byte)], memSize[nameof(MemorySize.FpuEnv14)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fnstenv_m28byte)], memSize[nameof(MemorySize.FpuEnv28)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstenv_m28byte)], memSize[nameof(MemorySize.FpuEnv28)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fnstcw_m2byte)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstcw_m2byte)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fld_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fxch_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fnop)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstpnce_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fchs)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fabs)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ftst)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fxam)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fld1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fldl2t)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fldl2e)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fldpi)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fldlg2)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fldln2)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fldz)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.F2xm1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fyl2x)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fptan)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fpatan)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fxtract)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fprem1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fdecstp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fincstp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fprem)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fyl2xp1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsqrt)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsincos)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Frndint)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fscale)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsin)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcos)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fiadd_m32int)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fimul_m32int)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ficom_m32int)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ficomp_m32int)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fisub_m32int)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fisubr_m32int)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fidiv_m32int)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fidivr_m32int)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcmovb_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcmove_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcmovbe_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcmovu_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fucompp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fild_m32int)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fisttp_m32int)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fist_m32int)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fistp_m32int)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fld_m80fp)], memSize[nameof(MemorySize.Float80)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstp_m80fp)], memSize[nameof(MemorySize.Float80)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcmovnb_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcmovne_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcmovnbe_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcmovnu_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fneni)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Feni)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fndisi)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fdisi)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fnclex)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fclex)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fninit)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Finit)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fnsetpm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsetpm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Frstpm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fucomi_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcomi_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fadd_m64fp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fmul_m64fp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcom_m64fp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcomp_m64fp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsub_m64fp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsubr_m64fp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fdiv_m64fp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fdivr_m64fp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fadd_sti_st0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fmul_sti_st0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcom_st0_sti_DCD0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcomp_st0_sti_DCD8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsubr_sti_st0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsub_sti_st0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fdivr_sti_st0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fdiv_sti_st0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fld_m64fp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fisttp_m64int)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fst_m64fp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstp_m64fp)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Frstor_m94byte)], memSize[nameof(MemorySize.FpuState94)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Frstor_m108byte)], memSize[nameof(MemorySize.FpuState108)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fnsave_m94byte)], memSize[nameof(MemorySize.FpuState94)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsave_m94byte)], memSize[nameof(MemorySize.FpuState94)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fnsave_m108byte)], memSize[nameof(MemorySize.FpuState108)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsave_m108byte)], memSize[nameof(MemorySize.FpuState108)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fnstsw_m2byte)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstsw_m2byte)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ffree_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fxch_st0_sti_DDC8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fst_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstp_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fucom_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fucomp_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fiadd_m16int)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fimul_m16int)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ficom_m16int)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ficomp_m16int)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fisub_m16int)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fisubr_m16int)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fidiv_m16int)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fidivr_m16int)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Faddp_sti_st0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fmulp_sti_st0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcomp_st0_sti_DED0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcompp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsubrp_sti_st0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fsubp_sti_st0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fdivrp_sti_st0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fdivp_sti_st0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fild_m16int)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fisttp_m16int)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fist_m16int)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fistp_m16int)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fbld_m80bcd)], memSize[nameof(MemorySize.Bcd)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fild_m64int)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fbstp_m80bcd)], memSize[nameof(MemorySize.Bcd)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fistp_m64int)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ffreep_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fxch_st0_sti_DFC8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstp_sti_DFD0)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstp_sti_DFD8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fnstsw_AX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstsw_AX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstdw_AX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fstsg_AX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fucomip_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fcomip_st0_sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loopne_rel8_16_CX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loopne_rel8_32_CX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loopne_rel8_16_ECX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loopne_rel8_32_ECX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loopne_rel8_64_ECX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loopne_rel8_16_RCX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loopne_rel8_64_RCX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loope_rel8_16_CX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loope_rel8_32_CX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loope_rel8_16_ECX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loope_rel8_32_ECX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loope_rel8_64_ECX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loope_rel8_16_RCX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loope_rel8_64_RCX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loop_rel8_16_CX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loop_rel8_32_CX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loop_rel8_16_ECX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loop_rel8_32_ECX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loop_rel8_64_ECX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loop_rel8_16_RCX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loop_rel8_64_RCX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jcxz_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jcxz_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jecxz_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jecxz_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jecxz_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jrcxz_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jrcxz_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.In_AL_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.In_AX_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.In_EAX_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Out_imm8_AL)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Out_imm8_AX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Out_imm8_EAX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Call_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Call_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Call_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_ptr1616)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_ptr1632)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_rel8_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_rel8_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_rel8_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.In_AL_DX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.In_AX_DX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.In_EAX_DX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Out_DX_AL)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Out_DX_AX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Out_DX_EAX)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Int1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Hlt)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_rm8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_rm8_imm8_F6r1)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Not_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Neg_rm8)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mul_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_rm8)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Div_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Idiv_rm8)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_rm16_imm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_rm64_imm32)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_rm16_imm16_F7r1)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_rm32_imm32_F7r1)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Test_rm64_imm32_F7r1)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Not_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Not_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Not_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Neg_rm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Neg_rm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Neg_rm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mul_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mul_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mul_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_rm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_rm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_rm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Div_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Div_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Div_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Idiv_rm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Idiv_rm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Idiv_rm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Clc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Stc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cli)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sti)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cld)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Std)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Inc_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Dec_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Inc_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Inc_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Inc_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Dec_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Dec_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Dec_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Call_rm16)], memSize[nameof(MemorySize.WordOffset)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Call_rm32)], memSize[nameof(MemorySize.DwordOffset)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Call_rm64)], memSize[nameof(MemorySize.QwordOffset)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Call_m1616)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Call_m1632)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Call_m1664)], memSize[nameof(MemorySize.SegPtr64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_rm16)], memSize[nameof(MemorySize.WordOffset)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_rm32)], memSize[nameof(MemorySize.DwordOffset)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_rm64)], memSize[nameof(MemorySize.QwordOffset)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_m1616)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_m1632)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmp_m1664)], memSize[nameof(MemorySize.SegPtr64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Push_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Push_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Push_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sldt_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sldt_r32m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sldt_r64m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Str_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Str_r32m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Str_r64m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lldt_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lldt_r32m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lldt_r64m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ltr_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ltr_r32m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ltr_r64m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Verr_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Verr_r32m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Verr_r64m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Verw_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Verw_r32m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Verw_r64m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmpe_rm16)], memSize[nameof(MemorySize.WordOffset)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmpe_rm32)], memSize[nameof(MemorySize.DwordOffset)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sgdt_m1632_16)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sgdt_m1632)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sgdt_m1664)], memSize[nameof(MemorySize.Fword10)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sidt_m1632_16)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sidt_m1632)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sidt_m1664)], memSize[nameof(MemorySize.Fword10)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lgdt_m1632_16)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lgdt_m1632)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lgdt_m1664)], memSize[nameof(MemorySize.Fword10)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lidt_m1632_16)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lidt_m1632)], memSize[nameof(MemorySize.Fword6)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lidt_m1664)], memSize[nameof(MemorySize.Fword10)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Smsw_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Smsw_r32m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Smsw_r64m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rstorssp_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lmsw_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lmsw_r32m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lmsw_r64m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Invlpg_m)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Enclv)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmcall)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmlaunch)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmresume)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmxoff)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pconfig)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Monitorw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Monitord)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Monitorq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mwait)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Clac)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Stac)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Encls)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xgetbv)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsetbv)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmfunc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xend)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xtest)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Enclu)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmrunw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmrund)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmrunq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmmcall)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmloadw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmloadd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmloadq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmsavew)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmsaved)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmsaveq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Stgi)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Clgi)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Skinit)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Invlpgaw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Invlpgad)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Invlpgaq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setssbsy)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Saveprevssp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdpkru)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wrpkru)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Swapgs)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdtscp)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Monitorxw)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Monitorxd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Monitorxq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mcommit)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mwaitx)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Clzerow)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Clzerod)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Clzeroq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdpru)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lar_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lar_r32_r32m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lar_r64_r64m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lsl_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lsl_r32_r32m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lsl_r64_r64m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loadallreset286)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loadall286)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Syscall)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Clts)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Loadall386)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sysretd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sysretq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Invd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wbinvd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wbnoinvd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cl1invmb)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ud2)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F0D)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F0D)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F0D)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Prefetch_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Prefetchw_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Prefetchwt1_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Femms)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Umov_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Umov_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Umov_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Umov_r8_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Umov_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Umov_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movups_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovups_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovups_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovups_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovups_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovups_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movupd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovupd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovupd_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovupd_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovupd_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovupd_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovss_xmm_xmm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovss_xmm_m32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovss_xmm_k1z_xmm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovss_xmm_k1z_m32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovsd_xmm_xmm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovsd_xmm_m64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovsd_xmm_k1z_m64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movups_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovups_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovups_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovups_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovups_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovups_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movupd_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovupd_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovupd_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovupd_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovupd_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovupd_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movss_xmmm32_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovss_xmm_xmm_xmm_0F11)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovss_m32_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovss_xmm_k1z_xmm_xmm_0F11)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovss_m32_k1_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsd_xmmm64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovsd_xmm_xmm_xmm_0F11)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovsd_m64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovsd_xmm_k1z_xmm_xmm_0F11)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovsd_m64_k1_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movhlps_xmm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movlps_xmm_m64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovhlps_xmm_xmm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovlps_xmm_xmm_m64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovhlps_xmm_xmm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovlps_xmm_xmm_m64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movlpd_xmm_m64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovlpd_xmm_xmm_m64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovlpd_xmm_xmm_m64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsldup_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovsldup_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovsldup_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovsldup_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovsldup_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovsldup_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movddup_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovddup_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovddup_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovddup_xmm_k1z_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovddup_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovddup_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movlps_m64_xmm)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovlps_m64_xmm)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovlps_m64_xmm)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movlpd_m64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovlpd_m64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovlpd_m64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Unpcklps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vunpcklps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vunpcklps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vunpcklps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vunpcklps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vunpcklps_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Unpcklpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vunpcklpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vunpcklpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vunpcklpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vunpcklpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vunpcklpd_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Unpckhps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vunpckhps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vunpckhps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vunpckhps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vunpckhps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vunpckhps_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Unpckhpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vunpckhpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vunpckhpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vunpckhpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vunpckhpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vunpckhpd_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Movlhps_xmm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovlhps_xmm_xmm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovlhps_xmm_xmm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movhps_xmm_m64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovhps_xmm_xmm_m64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovhps_xmm_xmm_m64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movhpd_xmm_m64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovhpd_xmm_xmm_m64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovhpd_xmm_xmm_m64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movshdup_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovshdup_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovshdup_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovshdup_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovshdup_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovshdup_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movhps_m64_xmm)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovhps_m64_xmm)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovhps_m64_xmm)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movhpd_m64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovhpd_m64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovhpd_m64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F18)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F18)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F18)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F19)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F19)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F19)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F1A)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F1A)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F1A)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F1B)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F1B)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F1B)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F1C)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F1C)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F1C)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F1D)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F1D)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F1D)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F1E)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F1E)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F1E)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm16_r16_0F1F)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm32_r32_0F1F)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.ReservedNop_rm64_r64_0F1F)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Prefetchnta_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Prefetcht0_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Prefetcht1_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Prefetcht2_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndldx_bnd_mib)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndmov_bnd_bndm64)], memSize[nameof(MemorySize.Bnd32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndmov_bnd_bndm128)], memSize[nameof(MemorySize.Bnd64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndcl_bnd_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndcl_bnd_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndcu_bnd_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndcu_bnd_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndstx_mib_bnd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndmov_bndm64_bnd)], memSize[nameof(MemorySize.Bnd32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndmov_bndm128_bnd)], memSize[nameof(MemorySize.Bnd64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndmk_bnd_m32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndmk_bnd_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndcn_bnd_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bndcn_bnd_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cldemote_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdsspd_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdsspq_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Endbr64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Endbr32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Nop_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Nop_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Nop_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r32_cr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r64_cr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r32_dr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r64_dr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_cr_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_cr_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_dr_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_dr_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_r32_tr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mov_tr_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movaps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovaps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovaps_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovaps_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovaps_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovaps_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movapd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovapd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovapd_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovapd_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovapd_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovapd_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movaps_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovaps_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovaps_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovaps_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovaps_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovaps_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movapd_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovapd_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovapd_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovapd_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovapd_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovapd_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtpi2ps_xmm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtpi2pd_xmm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtsi2ss_xmm_rm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtsi2ss_xmm_rm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtsi2ss_xmm_xmm_rm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtsi2ss_xmm_xmm_rm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtsi2ss_xmm_xmm_rm32_er)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtsi2ss_xmm_xmm_rm64_er)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtsi2sd_xmm_rm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtsi2sd_xmm_rm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtsi2sd_xmm_xmm_rm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtsi2sd_xmm_xmm_rm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtsi2sd_xmm_xmm_rm32_er)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtsi2sd_xmm_xmm_rm64_er)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movntps_m128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovntps_m128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovntps_m256_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovntps_m128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovntps_m256_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovntps_m512_zmm)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movntpd_m128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovntpd_m128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovntpd_m256_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovntpd_m128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovntpd_m256_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovntpd_m512_zmm)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movntss_m32_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movntsd_m64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvttps2pi_mm_xmmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvttpd2pi_mm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvttss2si_r32_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvttss2si_r64_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvttss2si_r32_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvttss2si_r64_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvttss2si_r32_xmmm32_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvttss2si_r64_xmmm32_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvttsd2si_r32_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvttsd2si_r64_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvttsd2si_r32_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvttsd2si_r64_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvttsd2si_r32_xmmm64_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvttsd2si_r64_xmmm64_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtps2pi_mm_xmmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtpd2pi_mm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtss2si_r32_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtss2si_r64_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtss2si_r32_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtss2si_r64_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtss2si_r32_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtss2si_r64_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtsd2si_r32_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtsd2si_r64_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtsd2si_r32_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtsd2si_r64_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtsd2si_r32_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtsd2si_r64_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ucomiss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vucomiss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vucomiss_xmm_xmmm32_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ucomisd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vucomisd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vucomisd_xmm_xmmm64_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Comiss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Comisd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcomiss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcomisd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcomiss_xmm_xmmm32_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcomisd_xmm_xmmm64_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wrmsr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdtsc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdmsr)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdpmc)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sysenter)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sysexitd)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sysexitq)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Getsec)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovo_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovo_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovo_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovno_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovno_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovno_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovb_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovb_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovb_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovae_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovae_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovae_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmove_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmove_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmove_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovne_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovne_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovne_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovbe_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovbe_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovbe_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmova_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmova_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmova_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovs_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovs_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovs_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovns_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovns_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovns_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovp_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovp_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovp_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovnp_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovnp_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovnp_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovl_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovl_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovl_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovge_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovge_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovge_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovle_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovle_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovle_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovg_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovg_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmovg_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kandw_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kandq_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kandb_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kandd_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kandnw_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kandnq_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kandnb_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kandnd_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Knotw_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Knotq_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Knotb_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Knotd_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Korw_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Korq_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Korb_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kord_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kxnorw_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kxnorq_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kxnorb_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kxnord_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kxorw_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kxorq_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kxorb_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kxord_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kaddw_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kaddq_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kaddb_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kaddd_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kunpckwd_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kunpckdq_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kunpckbw_k_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movmskps_r32_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movmskps_r64_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovmskps_r32_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovmskps_r64_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovmskps_r32_ymm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovmskps_r64_ymm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movmskpd_r32_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movmskpd_r64_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovmskpd_r32_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovmskpd_r64_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovmskpd_r32_ymm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovmskpd_r64_ymm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sqrtps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vsqrtps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vsqrtps_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vsqrtps_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vsqrtps_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vsqrtps_zmm_k1z_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Sqrtpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vsqrtpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vsqrtpd_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vsqrtpd_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vsqrtpd_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vsqrtpd_zmm_k1z_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Sqrtss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vsqrtss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sqrtsd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vsqrtsd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vsqrtsd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rsqrtps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vrsqrtps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vrsqrtps_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rsqrtss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vrsqrtss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcpps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vrcpps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vrcpps_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rcpss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vrcpss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Andps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vandps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vandps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vandps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vandps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vandps_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Andpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vandpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vandpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vandpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vandpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vandpd_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Andnps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vandnps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vandnps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vandnps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vandnps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vandnps_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Andnpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vandnpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vandnpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vandnpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vandnpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vandnpd_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Orps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vorps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vorps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vorps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vorps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vorps_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Orpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vorpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vorpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vorpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vorpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vorpd_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Xorps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vxorps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vxorps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vxorps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vxorps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vxorps_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Xorpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vxorpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vxorpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vxorpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vxorpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vxorpd_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Addps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaddps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaddps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaddps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vaddps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vaddps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Addpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaddpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaddpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaddpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vaddpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vaddpd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Addss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaddss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaddss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Addsd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaddsd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaddsd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mulps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmulps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmulps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmulps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vmulps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vmulps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Mulpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmulpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmulpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmulpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vmulpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vmulpd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Mulss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmulss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmulss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mulsd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmulsd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmulsd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtps2pd_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtps2pd_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtps2pd_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtps2pd_xmm_k1z_xmmm64b32)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Broadcast64_Float32)]),
				(code[nameof(Code.EVEX_Vcvtps2pd_ymm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vcvtps2pd_zmm_k1z_ymmm256b32_sae)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.Cvtpd2ps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtpd2ps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtpd2ps_xmm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtpd2ps_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vcvtpd2ps_xmm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vcvtpd2ps_ymm_k1z_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Cvtss2sd_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtss2sd_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtss2sd_xmm_k1z_xmm_xmmm32_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtsd2ss_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtsd2ss_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtsd2ss_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvtdq2ps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtdq2ps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtdq2ps_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtdq2ps_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vcvtdq2ps_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vcvtdq2ps_zmm_k1z_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vcvtqq2ps_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vcvtqq2ps_xmm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vcvtqq2ps_ymm_k1z_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Cvtps2dq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtps2dq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtps2dq_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtps2dq_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vcvtps2dq_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vcvtps2dq_zmm_k1z_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Cvttps2dq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvttps2dq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvttps2dq_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvttps2dq_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vcvttps2dq_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vcvttps2dq_zmm_k1z_zmmm512b32_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Subps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vsubps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vsubps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vsubps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vsubps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vsubps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Subpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vsubpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vsubpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vsubpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vsubpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vsubpd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Subss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vsubss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vsubss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Subsd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vsubsd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vsubsd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Minps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vminps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vminps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vminps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vminps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vminps_zmm_k1z_zmm_zmmm512b32_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Minpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vminpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vminpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vminpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vminpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vminpd_zmm_k1z_zmm_zmmm512b64_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Minss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vminss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vminss_xmm_k1z_xmm_xmmm32_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Minsd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vminsd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vminsd_xmm_k1z_xmm_xmmm64_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Divps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vdivps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vdivps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vdivps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vdivps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vdivps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Divpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vdivpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vdivpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vdivpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vdivpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vdivpd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Divss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vdivss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vdivss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Divsd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vdivsd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vdivsd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Maxps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaxps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaxps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmaxps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vmaxps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vmaxps_zmm_k1z_zmm_zmmm512b32_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Maxpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaxpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaxpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmaxpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vmaxpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vmaxpd_zmm_k1z_zmm_zmmm512b64_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Maxss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaxss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmaxss_xmm_k1z_xmm_xmmm32_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Maxsd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaxsd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmaxsd_xmm_k1z_xmm_xmmm64_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Punpcklbw_mm_mmm32)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Punpcklbw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpcklbw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpcklbw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpcklbw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpcklbw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpcklbw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Punpcklwd_mm_mmm32)], memSize[nameof(MemorySize.Packed32_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Punpcklwd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpcklwd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpcklwd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpcklwd_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpcklwd_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpcklwd_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Punpckldq_mm_mmm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Punpckldq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpckldq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpckldq_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpckldq_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpunpckldq_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpunpckldq_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.Packsswb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Packsswb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpacksswb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpacksswb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpacksswb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpacksswb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpacksswb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpgtb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpgtb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpgtb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpgtb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpgtb_k_k1_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpgtb_k_k1_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpgtb_k_k1_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpgtw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpgtw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpgtw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpgtw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpgtw_k_k1_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpgtw_k_k1_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpgtw_k_k1_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpgtd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpgtd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpgtd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpgtd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpgtd_k_k1_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpcmpgtd_k_k1_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpcmpgtd_k_k1_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.Packuswb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Packuswb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpackuswb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpackuswb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpackuswb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpackuswb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpackuswb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Punpckhbw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Punpckhbw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpckhbw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpckhbw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpckhbw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpckhbw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpckhbw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Punpckhwd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Punpckhwd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpckhwd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpckhwd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpckhwd_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpckhwd_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpckhwd_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Punpckhdq_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Punpckhdq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpckhdq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpckhdq_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpckhdq_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpunpckhdq_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpunpckhdq_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.Packssdw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Packssdw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpackssdw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpackssdw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpackssdw_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpackssdw_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpackssdw_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.Punpcklqdq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpcklqdq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpcklqdq_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpcklqdq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpunpcklqdq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpunpcklqdq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Punpckhqdq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpckhqdq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpunpckhqdq_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpunpckhqdq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpunpckhqdq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpunpckhqdq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Movd_mm_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movq_mm_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movd_xmm_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movq_xmm_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovd_xmm_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovq_xmm_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovd_xmm_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovq_xmm_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movq_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movdqa_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovdqa_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovdqa_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqa32_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqa32_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqa32_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqa64_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqa64_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqa64_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movdqu_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovdqu_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovdqu_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu32_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu32_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu32_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu64_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu64_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu64_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu8_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu8_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu8_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu16_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu16_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu16_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pshufw_mm_mmm64_imm8)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pshufd_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpshufd_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpshufd_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshufd_xmm_k1z_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpshufd_ymm_k1z_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpshufd_zmm_k1z_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.Pshufhw_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpshufhw_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpshufhw_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshufhw_xmm_k1z_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshufhw_ymm_k1z_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshufhw_zmm_k1z_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pshuflw_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpshuflw_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpshuflw_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshuflw_xmm_k1z_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshuflw_ymm_k1z_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshuflw_zmm_k1z_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrlw_mm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrlw_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrlw_xmm_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrlw_ymm_ymm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlw_xmm_k1z_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlw_ymm_k1z_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlw_zmm_k1z_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psraw_mm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psraw_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsraw_xmm_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsraw_ymm_ymm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsraw_xmm_k1z_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsraw_ymm_k1z_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsraw_zmm_k1z_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psllw_mm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psllw_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsllw_xmm_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsllw_ymm_ymm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllw_xmm_k1z_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllw_ymm_k1z_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllw_zmm_k1z_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vprord_xmm_k1z_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vprord_ymm_k1z_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vprord_zmm_k1z_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vprorq_xmm_k1z_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vprorq_ymm_k1z_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vprorq_zmm_k1z_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.EVEX_Vprold_xmm_k1z_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vprold_ymm_k1z_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vprold_zmm_k1z_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vprolq_xmm_k1z_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vprolq_ymm_k1z_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vprolq_zmm_k1z_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Psrld_mm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrld_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrld_xmm_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrld_ymm_ymm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrld_xmm_k1z_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpsrld_ymm_k1z_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpsrld_zmm_k1z_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.Psrad_mm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrad_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrad_xmm_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrad_ymm_ymm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrad_xmm_k1z_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpsrad_ymm_k1z_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpsrad_zmm_k1z_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vpsraq_xmm_k1z_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpsraq_ymm_k1z_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpsraq_zmm_k1z_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Pslld_mm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pslld_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpslld_xmm_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpslld_ymm_ymm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpslld_xmm_k1z_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpslld_ymm_k1z_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpslld_zmm_k1z_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.Psrlq_mm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrlq_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrlq_xmm_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrlq_ymm_ymm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlq_xmm_k1z_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpsrlq_ymm_k1z_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpsrlq_zmm_k1z_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Psrldq_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrldq_xmm_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrldq_ymm_ymm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrldq_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrldq_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrldq_zmm_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psllq_mm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psllq_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsllq_xmm_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsllq_ymm_ymm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllq_xmm_k1z_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpsllq_ymm_k1z_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpsllq_zmm_k1z_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Pslldq_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpslldq_xmm_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpslldq_ymm_ymm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpslldq_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpslldq_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpslldq_zmm_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpeqb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpeqb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpeqb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpeqb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpeqb_k_k1_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpeqb_k_k1_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpeqb_k_k1_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpeqw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpeqw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpeqw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpeqw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpeqw_k_k1_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpeqw_k_k1_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpeqw_k_k1_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpeqd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpeqd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpeqd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpeqd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpeqd_k_k1_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpcmpeqd_k_k1_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpcmpeqd_k_k1_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.Emms)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vzeroupper)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vzeroall)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmread_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmread_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvttps2udq_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vcvttps2udq_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vcvttps2udq_zmm_k1z_zmmm512b32_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vcvttpd2udq_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vcvttpd2udq_xmm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vcvttpd2udq_ymm_k1z_zmmm512b64_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Extrq_xmm_imm8_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvttps2uqq_xmm_k1z_xmmm64b32)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Broadcast64_Float32)]),
				(code[nameof(Code.EVEX_Vcvttps2uqq_ymm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vcvttps2uqq_zmm_k1z_ymmm256b32_sae)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vcvttpd2uqq_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vcvttpd2uqq_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vcvttpd2uqq_zmm_k1z_zmmm512b64_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vcvttss2usi_r32_xmmm32_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvttss2usi_r64_xmmm32_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Insertq_xmm_xmm_imm8_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvttsd2usi_r32_xmmm64_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvttsd2usi_r64_xmmm64_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmwrite_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmwrite_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtps2udq_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vcvtps2udq_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vcvtps2udq_zmm_k1z_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vcvtpd2udq_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vcvtpd2udq_xmm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vcvtpd2udq_ymm_k1z_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Extrq_xmm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtps2uqq_xmm_k1z_xmmm64b32)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Broadcast64_Float32)]),
				(code[nameof(Code.EVEX_Vcvtps2uqq_ymm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vcvtps2uqq_zmm_k1z_ymmm256b32_er)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vcvtpd2uqq_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vcvtpd2uqq_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vcvtpd2uqq_zmm_k1z_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vcvtss2usi_r32_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtss2usi_r64_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Insertq_xmm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtsd2usi_r32_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtsd2usi_r64_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvttps2qq_xmm_k1z_xmmm64b32)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Broadcast64_Float32)]),
				(code[nameof(Code.EVEX_Vcvttps2qq_ymm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vcvttps2qq_zmm_k1z_ymmm256b32_sae)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vcvttpd2qq_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vcvttpd2qq_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vcvttpd2qq_zmm_k1z_zmmm512b64_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vcvtudq2pd_xmm_k1z_xmmm64b32)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Broadcast64_UInt32)]),
				(code[nameof(Code.EVEX_Vcvtudq2pd_ymm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vcvtudq2pd_zmm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vcvtuqq2pd_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vcvtuqq2pd_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vcvtuqq2pd_zmm_k1z_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.EVEX_Vcvtudq2ps_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vcvtudq2ps_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vcvtudq2ps_zmm_k1z_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vcvtuqq2ps_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vcvtuqq2ps_xmm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vcvtuqq2ps_ymm_k1z_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.EVEX_Vcvtps2qq_xmm_k1z_xmmm64b32)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Broadcast64_Float32)]),
				(code[nameof(Code.EVEX_Vcvtps2qq_ymm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vcvtps2qq_zmm_k1z_ymmm256b32_er)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vcvtpd2qq_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vcvtpd2qq_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vcvtpd2qq_zmm_k1z_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vcvtusi2ss_xmm_xmm_rm32_er)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtusi2ss_xmm_xmm_rm64_er)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtusi2sd_xmm_xmm_rm32_er)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtusi2sd_xmm_xmm_rm64_er)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Haddpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vhaddpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vhaddpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Haddps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vhaddps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vhaddps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Hsubpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vhsubpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vhsubpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Hsubps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vhsubps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vhsubps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movd_rm32_mm)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movq_rm64_mm)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movd_rm32_xmm)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movq_rm64_xmm)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovd_rm32_xmm)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovq_rm64_xmm)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovd_rm32_xmm)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovq_rm64_xmm)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movq_xmm_xmmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovq_xmm_xmmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovq_xmm_xmmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movq_mmm64_mm)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movdqa_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovdqa_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovdqa_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqa32_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqa32_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqa32_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqa64_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqa64_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqa64_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movdqu_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovdqu_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovdqu_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu32_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu32_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu32_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu64_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu64_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu64_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu8_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu8_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu8_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu16_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu16_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovdqu16_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jo_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jo_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jo_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jno_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jno_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jno_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jb_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jb_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jb_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jae_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jae_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jae_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Je_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Je_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Je_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jne_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jne_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jne_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jbe_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jbe_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jbe_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ja_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ja_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ja_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Js_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Js_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Js_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jns_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jns_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jns_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jp_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jp_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jp_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jnp_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jnp_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jnp_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jl_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jl_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jl_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jge_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jge_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jge_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jle_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jle_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jle_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jg_rel16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jg_rel32_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jg_rel32_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Seto_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setno_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setb_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setae_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sete_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setne_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setbe_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Seta_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sets_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setns_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setp_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setnp_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setl_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setge_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setle_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Setg_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovw_k_km16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovq_k_km64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovb_k_km8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovd_k_km32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovw_m16_k)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovq_m64_k)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovb_m8_k)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovd_m32_k)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovw_k_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovb_k_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovd_k_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovq_k_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovw_r32_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovb_r32_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovd_r32_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kmovq_r64_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kortestw_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kortestq_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kortestb_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kortestd_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Ktestw_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Ktestq_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Ktestb_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Ktestd_k_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushw_FS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushd_FS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushq_FS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popw_FS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popd_FS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popq_FS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cpuid)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bt_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bt_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bt_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shld_rm16_r16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shld_rm32_r32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shld_rm64_r64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shld_rm16_r16_CL)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shld_rm32_r32_CL)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shld_rm64_r64_CL)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Montmul_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Montmul_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Montmul_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsha1_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsha1_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsha1_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsha256_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsha256_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsha256_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xbts_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xbts_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xstore_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xstore_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xstore_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptEcb_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptEcb_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptEcb_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptCbc_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptCbc_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptCbc_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptCtr_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptCtr_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptCtr_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptCfb_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptCfb_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptCfb_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptOfb_16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptOfb_32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XcryptOfb_64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ibts_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ibts_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpxchg486_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpxchg486_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpxchg486_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushw_GS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushd_GS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pushq_GS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popw_GS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popd_GS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popq_GS)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rsm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bts_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bts_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bts_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shrd_rm16_r16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shrd_rm32_r32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shrd_rm64_r64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shrd_rm16_r16_CL)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shrd_rm32_r32_CL)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shrd_rm64_r64_CL)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fxsave_m512byte)], memSize[nameof(MemorySize.Fxsave_512Byte)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fxsave64_m512byte)], memSize[nameof(MemorySize.Fxsave64_512Byte)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdfsbase_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdfsbase_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fxrstor_m512byte)], memSize[nameof(MemorySize.Fxsave_512Byte)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Fxrstor64_m512byte)], memSize[nameof(MemorySize.Fxsave64_512Byte)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdgsbase_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdgsbase_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ldmxcsr_m32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wrfsbase_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wrfsbase_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vldmxcsr_m32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Stmxcsr_m32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wrgsbase_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wrgsbase_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vstmxcsr_m32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsave_mem)], memSize[nameof(MemorySize.Xsave)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsave64_mem)], memSize[nameof(MemorySize.Xsave64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ptwrite_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ptwrite_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xrstor_mem)], memSize[nameof(MemorySize.Xsave)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xrstor64_mem)], memSize[nameof(MemorySize.Xsave64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Incsspd_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Incsspq_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsaveopt_mem)], memSize[nameof(MemorySize.Xsave)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsaveopt64_mem)], memSize[nameof(MemorySize.Xsave64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Clwb_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Tpause_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Tpause_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Clrssbsy_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Umonitor_r16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Umonitor_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Umonitor_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Umwait_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Umwait_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Clflush_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Clflushopt_m8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lfence_E9)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lfence_EA)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lfence_EB)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lfence_EC)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lfence_ED)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lfence_EE)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lfence_EF)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mfence_F1)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mfence_F2)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mfence_F3)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mfence_F4)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mfence_F5)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mfence_F6)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mfence_F7)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sfence)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sfence_F9)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sfence_FA)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sfence_FB)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sfence_FC)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sfence_FD)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sfence_FE)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sfence_FF)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcommit)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_r16_rm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_r32_rm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Imul_r64_rm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpxchg_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpxchg_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpxchg_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpxchg_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lss_r16_m1616)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lss_r32_m1632)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lss_r64_m1664)], memSize[nameof(MemorySize.SegPtr64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Btr_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Btr_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Btr_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lfs_r16_m1616)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lfs_r32_m1632)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lfs_r64_m1664)], memSize[nameof(MemorySize.SegPtr64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lgs_r16_m1616)], memSize[nameof(MemorySize.SegPtr16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lgs_r32_m1632)], memSize[nameof(MemorySize.SegPtr32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lgs_r64_m1664)], memSize[nameof(MemorySize.SegPtr64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movzx_r16_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movzx_r32_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movzx_r64_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movzx_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movzx_r32_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movzx_r64_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmpe_disp16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Jmpe_disp32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popcnt_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popcnt_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Popcnt_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ud1_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ud1_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ud1_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bt_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bt_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bt_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bts_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bts_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bts_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Btr_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Btr_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Btr_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Btc_rm16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Btc_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Btc_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Btc_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Btc_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Btc_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bsf_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bsf_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bsf_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Tzcnt_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Tzcnt_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Tzcnt_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bsr_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bsr_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bsr_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lzcnt_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lzcnt_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Lzcnt_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsx_r16_rm8)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsx_r32_rm8)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsx_r64_rm8)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsx_r16_rm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsx_r32_rm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movsx_r64_rm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xadd_rm8_r8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xadd_rm16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xadd_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xadd_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpps_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Cmppd_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Cmpss_xmm_xmmm32_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpsd_xmm_xmmm64_imm8)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movnti_m32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movnti_m64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pinsrw_mm_r32m16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pinsrw_mm_r64m16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pinsrw_xmm_r32m16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pinsrw_xmm_r64m16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpinsrw_xmm_xmm_r32m16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpinsrw_xmm_xmm_r64m16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpinsrw_xmm_xmm_r32m16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpinsrw_xmm_xmm_r64m16_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pextrw_r32_mm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pextrw_r64_mm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pextrw_r32_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pextrw_r64_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpextrw_r32_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpextrw_r64_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpextrw_r32_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpextrw_r64_xmm_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Shufps_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vshufps_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vshufps_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vshufps_xmm_k1z_xmm_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vshufps_ymm_k1z_ymm_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vshufps_zmm_k1z_zmm_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Shufpd_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vshufpd_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vshufpd_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vshufpd_xmm_k1z_xmm_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vshufpd_ymm_k1z_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vshufpd_zmm_k1z_zmm_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Cmpxchg8b_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cmpxchg16b_m128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xrstors_mem)], memSize[nameof(MemorySize.Xsave)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xrstors64_mem)], memSize[nameof(MemorySize.Xsave64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsavec_mem)], memSize[nameof(MemorySize.Xsave)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsavec64_mem)], memSize[nameof(MemorySize.Xsave64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsaves_mem)], memSize[nameof(MemorySize.Xsave)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Xsaves64_mem)], memSize[nameof(MemorySize.Xsave64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmptrld_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmclear_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmxon_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdrand_r16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdrand_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdrand_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Vmptrst_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdseed_r16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdseed_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdseed_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdpid_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Rdpid_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bswap_r16)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bswap_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Bswap_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Addsubpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaddsubpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaddsubpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Addsubps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaddsubps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaddsubps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrlw_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrlw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrlw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrlw_ymm_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlw_ymm_k1z_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlw_zmm_k1z_zmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrld_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrld_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrld_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrld_ymm_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrld_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrld_ymm_k1z_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrld_zmm_k1z_zmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrlq_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrlq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrlq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrlq_ymm_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlq_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlq_ymm_k1z_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlq_zmm_k1z_zmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddq_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddq_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpaddq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpaddq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Pmullw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmullw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmullw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmullw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmullw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmullw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmullw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movq_xmmm64_xmm)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovq_xmmm64_xmm)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovq_xmmm64_xmm)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movq2dq_xmm_mm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movdq2q_mm_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovmskb_r32_mm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovmskb_r64_mm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovmskb_r32_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovmskb_r64_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovmskb_r32_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovmskb_r64_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovmskb_r32_ymm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovmskb_r64_ymm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubusb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubusb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubusb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubusb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubusb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubusb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubusb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubusw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubusw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubusw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubusw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubusw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubusw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubusw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pminub_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pminub_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpminub_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpminub_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminub_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminub_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminub_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pand_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pand_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpand_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpand_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpandd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpandd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpandd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpandq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpandq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpandq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Paddusb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddusb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddusb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddusb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddusb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddusb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddusb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddusw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddusw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddusw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddusw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddusw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddusw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddusw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmaxub_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmaxub_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaxub_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaxub_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxub_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxub_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxub_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pandn_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pandn_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpandn_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpandn_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpandnd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpandnd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpandnd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpandnq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpandnq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpandnq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Pavgb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pavgb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpavgb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpavgb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpavgb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpavgb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpavgb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psraw_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psraw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsraw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsraw_ymm_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsraw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsraw_ymm_k1z_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsraw_zmm_k1z_zmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrad_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psrad_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrad_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrad_ymm_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrad_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrad_ymm_k1z_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrad_zmm_k1z_zmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsraq_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsraq_ymm_k1z_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsraq_zmm_k1z_zmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pavgw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pavgw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpavgw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpavgw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpavgw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpavgw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpavgw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmulhuw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmulhuw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmulhuw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmulhuw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmulhuw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmulhuw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmulhuw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmulhw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmulhw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmulhw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmulhw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmulhw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmulhw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmulhw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Cvttpd2dq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvttpd2dq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvttpd2dq_xmm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvttpd2dq_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vcvttpd2dq_xmm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vcvttpd2dq_ymm_k1z_zmmm512b64_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Cvtdq2pd_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtdq2pd_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtdq2pd_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtdq2pd_xmm_k1z_xmmm64b32)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Broadcast64_Int32)]),
				(code[nameof(Code.EVEX_Vcvtdq2pd_ymm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vcvtdq2pd_zmm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vcvtqq2pd_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vcvtqq2pd_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vcvtqq2pd_zmm_k1z_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Cvtpd2dq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtpd2dq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtpd2dq_xmm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtpd2dq_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vcvtpd2dq_xmm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vcvtpd2dq_ymm_k1z_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Movntq_m64_mm)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movntdq_m128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovntdq_m128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovntdq_m256_ymm)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovntdq_m128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovntdq_m256_ymm)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovntdq_m512_zmm)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubsb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubsb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubsb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubsb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubsb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubsb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubsb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubsw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubsw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubsw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubsw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubsw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubsw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubsw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pminsw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pminsw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpminsw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpminsw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminsw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminsw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminsw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Por_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Por_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpor_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpor_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpord_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpord_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpord_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vporq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vporq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vporq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Paddsb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddsb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddsb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddsb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddsb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddsb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddsb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddsw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddsw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddsw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddsw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddsw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddsw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddsw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmaxsw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmaxsw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaxsw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaxsw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxsw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxsw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxsw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pxor_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pxor_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpxor_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpxor_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpxord_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpxord_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpxord_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpxorq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpxorq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpxorq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Lddqu_xmm_m128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vlddqu_xmm_m128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vlddqu_ymm_m256)], memSize[nameof(MemorySize.UInt256)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psllw_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psllw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsllw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsllw_ymm_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllw_ymm_k1z_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllw_zmm_k1z_zmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pslld_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pslld_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpslld_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpslld_ymm_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpslld_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpslld_ymm_k1z_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpslld_zmm_k1z_zmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psllq_mm_mmm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psllq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsllq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsllq_ymm_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllq_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllq_ymm_k1z_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllq_zmm_k1z_zmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmuludq_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmuludq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmuludq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmuludq_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmuludq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_2xUInt32)]),
				(code[nameof(Code.EVEX_Vpmuludq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_2xUInt32)]),
				(code[nameof(Code.EVEX_Vpmuludq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_2xUInt32)]),
				(code[nameof(Code.Pmaddwd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmaddwd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaddwd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaddwd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaddwd_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaddwd_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaddwd_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psadbw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psadbw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsadbw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsadbw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsadbw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsadbw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsadbw_zmm_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Maskmovq_rDI_mm_mm)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Maskmovdqu_rDI_xmm_xmm)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaskmovdqu_rDI_xmm_xmm)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpsubd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpsubd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.Psubq_mm_mmm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psubq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsubq_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsubq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpsubq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpsubq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Paddb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Paddd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpaddd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpaddd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpaddd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpaddd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.Ud0_r16_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ud0_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Ud0_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pshufb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pshufb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpshufb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpshufb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshufb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshufb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshufb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Phaddw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Phaddw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphaddw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphaddw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Phaddd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Phaddd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphaddd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphaddd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Phaddsw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Phaddsw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphaddsw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphaddsw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmaddubsw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmaddubsw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaddubsw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaddubsw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaddubsw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaddubsw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaddubsw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Phsubw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Phsubw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphsubw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphsubw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Phsubd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Phsubd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphsubd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphsubd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Phsubsw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Phsubsw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphsubsw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphsubsw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psignb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psignb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsignb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsignb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psignw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psignw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsignw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsignw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psignd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Psignd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsignd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsignd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmulhrsw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmulhrsw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmulhrsw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmulhrsw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmulhrsw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmulhrsw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmulhrsw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermilps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermilps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermilps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vpermilps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vpermilps_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.VEX_Vpermilpd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermilpd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermilpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vpermilpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vpermilpd_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vtestps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vtestps_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vtestpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vtestpd_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pblendvb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlvw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlvw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlvw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovuswb_xmmm64_k1z_xmm)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovuswb_xmmm128_k1z_ymm)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovuswb_ymmm256_k1z_zmm)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsravw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsravw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsravw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusdb_xmmm32_k1z_xmm)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusdb_xmmm64_k1z_ymm)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusdb_xmmm128_k1z_zmm)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllvw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllvw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllvw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusqb_xmmm16_k1z_xmm)], memSize[nameof(MemorySize.Packed16_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusqb_xmmm32_k1z_ymm)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusqb_xmmm64_k1z_zmm)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtph2ps_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_Float16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtph2ps_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtph2ps_xmm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_Float16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtph2ps_ymm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Float16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtph2ps_zmm_k1z_ymmm256_sae)], memSize[nameof(MemorySize.Packed256_Float16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusdw_xmmm64_k1z_xmm)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusdw_xmmm128_k1z_ymm)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusdw_ymmm256_k1z_zmm)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Blendvps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vprorvd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vprorvd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vprorvd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vprorvq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vprorvq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vprorvq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.EVEX_Vpmovusqw_xmmm32_k1z_xmm)], memSize[nameof(MemorySize.Packed32_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusqw_xmmm64_k1z_ymm)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusqw_xmmm128_k1z_zmm)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Blendvpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vprolvd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vprolvd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vprolvd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vprolvq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vprolvq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vprolvq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.EVEX_Vpmovusqd_xmmm64_k1z_xmm)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusqd_xmmm128_k1z_ymm)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovusqd_ymmm256_k1z_zmm)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vpermps_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vpermpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vpermpd_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Ptest_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vptest_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vptest_ymm_ymmm256)], memSize[nameof(MemorySize.UInt256)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vbroadcastss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vbroadcastss_ymm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastss_xmm_k1z_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastss_ymm_k1z_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastss_zmm_k1z_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vbroadcastsd_ymm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastf32x2_ymm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastf32x2_zmm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastsd_ymm_k1z_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastsd_zmm_k1z_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vbroadcastf128_ymm_m128)], memSize[nameof(MemorySize.Float128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastf32x4_ymm_k1z_m128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastf32x4_zmm_k1z_m128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastf64x2_ymm_k1z_m128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastf64x2_zmm_k1z_m128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastf32x8_zmm_k1z_m256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcastf64x4_zmm_k1z_m256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pabsb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pabsb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpabsb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpabsb_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpabsb_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpabsb_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpabsb_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pabsw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pabsw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpabsw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpabsw_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpabsw_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpabsw_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpabsw_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pabsd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pabsd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpabsd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpabsd_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpabsd_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpabsd_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpabsd_zmm_k1z_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vpabsq_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpabsq_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpabsq_zmm_k1z_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Pmovsxbw_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovsxbw_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovsxbw_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxbw_xmm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxbw_ymm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxbw_zmm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovswb_xmmm64_k1z_xmm)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovswb_xmmm128_k1z_ymm)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovswb_ymmm256_k1z_zmm)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovsxbd_xmm_xmmm32)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovsxbd_xmm_xmmm32)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovsxbd_ymm_xmmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxbd_xmm_k1z_xmmm32)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxbd_ymm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxbd_zmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsdb_xmmm32_k1z_xmm)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsdb_xmmm64_k1z_ymm)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsdb_xmmm128_k1z_zmm)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovsxbq_xmm_xmmm16)], memSize[nameof(MemorySize.Packed16_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovsxbq_xmm_xmmm16)], memSize[nameof(MemorySize.Packed16_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovsxbq_ymm_xmmm32)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxbq_xmm_k1z_xmmm16)], memSize[nameof(MemorySize.Packed16_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxbq_ymm_k1z_xmmm32)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxbq_zmm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsqb_xmmm16_k1z_xmm)], memSize[nameof(MemorySize.Packed16_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsqb_xmmm32_k1z_ymm)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsqb_xmmm64_k1z_zmm)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovsxwd_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovsxwd_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovsxwd_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxwd_xmm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxwd_ymm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxwd_zmm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsdw_xmmm64_k1z_xmm)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsdw_xmmm128_k1z_ymm)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsdw_ymmm256_k1z_zmm)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovsxwq_xmm_xmmm32)], memSize[nameof(MemorySize.Packed32_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovsxwq_xmm_xmmm32)], memSize[nameof(MemorySize.Packed32_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovsxwq_ymm_xmmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxwq_xmm_k1z_xmmm32)], memSize[nameof(MemorySize.Packed32_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxwq_ymm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxwq_zmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsqw_xmmm32_k1z_xmm)], memSize[nameof(MemorySize.Packed32_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsqw_xmmm64_k1z_ymm)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsqw_xmmm128_k1z_zmm)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovsxdq_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovsxdq_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovsxdq_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxdq_xmm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxdq_ymm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsxdq_zmm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsqd_xmmm64_k1z_xmm)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsqd_xmmm128_k1z_ymm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovsqd_ymmm256_k1z_zmm)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestmb_k_k1_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestmb_k_k1_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestmb_k_k1_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestmw_k_k1_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestmw_k_k1_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestmw_k_k1_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestnmb_k_k1_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestnmb_k_k1_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestnmb_k_k1_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestnmw_k_k1_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestnmw_k_k1_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestnmw_k_k1_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vptestmd_k_k1_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vptestmd_k_k1_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vptestmd_k_k1_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vptestmq_k_k1_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vptestmq_k_k1_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vptestmq_k_k1_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.EVEX_Vptestnmd_k_k1_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vptestnmd_k_k1_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vptestnmd_k_k1_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vptestnmq_k_k1_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vptestnmq_k_k1_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vptestnmq_k_k1_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Pmuldq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmuldq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmuldq_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmuldq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_2xInt32)]),
				(code[nameof(Code.EVEX_Vpmuldq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_2xInt32)]),
				(code[nameof(Code.EVEX_Vpmuldq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_2xInt32)]),
				(code[nameof(Code.EVEX_Vpmovm2b_xmm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovm2b_ymm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovm2b_zmm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovm2w_xmm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovm2w_ymm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovm2w_zmm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpeqq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpeqq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpeqq_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpeqq_k_k1_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpcmpeqq_k_k1_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpcmpeqq_k_k1_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.EVEX_Vpmovb2m_k_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovb2m_k_ymm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovb2m_k_zmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovw2m_k_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovw2m_k_ymm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovw2m_k_zmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movntdqa_xmm_m128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovntdqa_xmm_m128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmovntdqa_ymm_m256)], memSize[nameof(MemorySize.UInt256)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovntdqa_xmm_m128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovntdqa_ymm_m256)], memSize[nameof(MemorySize.UInt256)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vmovntdqa_zmm_m512)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastmb2q_xmm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastmb2q_ymm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastmb2q_zmm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Packusdw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpackusdw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpackusdw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpackusdw_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpackusdw_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpackusdw_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.VEX_Vmaskmovps_xmm_xmm_m128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaskmovps_ymm_ymm_m256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscalefps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vscalefps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vscalefps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vscalefpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vscalefpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vscalefpd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vmaskmovpd_xmm_xmm_m128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaskmovpd_ymm_ymm_m256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscalefss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscalefsd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaskmovps_m128_xmm_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaskmovps_m256_ymm_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaskmovpd_m128_xmm_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmaskmovpd_m256_ymm_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovzxbw_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovzxbw_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovzxbw_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxbw_xmm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxbw_ymm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxbw_zmm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovwb_xmmm64_k1z_xmm)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovwb_xmmm128_k1z_ymm)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovwb_ymmm256_k1z_zmm)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovzxbd_xmm_xmmm32)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovzxbd_xmm_xmmm32)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovzxbd_ymm_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxbd_xmm_k1z_xmmm32)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxbd_ymm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxbd_zmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovdb_xmmm32_k1z_xmm)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovdb_xmmm64_k1z_ymm)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovdb_xmmm128_k1z_zmm)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovzxbq_xmm_xmmm16)], memSize[nameof(MemorySize.Packed16_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovzxbq_xmm_xmmm16)], memSize[nameof(MemorySize.Packed16_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovzxbq_ymm_xmmm32)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxbq_xmm_k1z_xmmm16)], memSize[nameof(MemorySize.Packed16_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxbq_ymm_k1z_xmmm32)], memSize[nameof(MemorySize.Packed32_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxbq_zmm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovqb_xmmm16_k1z_xmm)], memSize[nameof(MemorySize.Packed16_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovqb_xmmm32_k1z_ymm)], memSize[nameof(MemorySize.Packed32_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovqb_xmmm64_k1z_zmm)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovzxwd_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovzxwd_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovzxwd_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxwd_xmm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxwd_ymm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxwd_zmm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovdw_xmmm64_k1z_xmm)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovdw_xmmm128_k1z_ymm)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovdw_ymmm256_k1z_zmm)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovzxwq_xmm_xmmm32)], memSize[nameof(MemorySize.Packed32_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovzxwq_xmm_xmmm32)], memSize[nameof(MemorySize.Packed32_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovzxwq_ymm_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxwq_xmm_k1z_xmmm32)], memSize[nameof(MemorySize.Packed32_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxwq_ymm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxwq_zmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovqw_xmmm32_k1z_xmm)], memSize[nameof(MemorySize.Packed32_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovqw_xmmm64_k1z_ymm)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovqw_xmmm128_k1z_zmm)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmovzxdq_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovzxdq_xmm_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmovzxdq_ymm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxdq_xmm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxdq_ymm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovzxdq_zmm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovqd_xmmm64_k1z_xmm)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovqd_xmmm128_k1z_ymm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovqd_ymmm256_k1z_zmm)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpermd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vpermq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpermq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Pcmpgtq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpgtq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpgtq_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpgtq_k_k1_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpcmpgtq_k_k1_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpcmpgtq_k_k1_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Pminsb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpminsb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpminsb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminsb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminsb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminsb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovm2d_xmm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovm2d_ymm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovm2d_zmm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovm2q_xmm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovm2q_ymm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovm2q_zmm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pminsd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpminsd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpminsd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminsd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpminsd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpminsd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vpminsq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpminsq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpminsq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.EVEX_Vpmovd2m_k_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovd2m_k_ymm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovd2m_k_zmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovq2m_k_xmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovq2m_k_ymm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmovq2m_k_zmm)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pminuw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpminuw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpminuw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminuw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminuw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminuw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastmw2d_xmm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastmw2d_ymm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastmw2d_zmm_k)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pminud_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpminud_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpminud_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpminud_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpminud_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpminud_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpminuq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpminuq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpminuq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Pmaxsb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaxsb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaxsb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxsb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxsb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxsb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmaxsd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaxsd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaxsd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxsd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpmaxsd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpmaxsd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vpmaxsq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpmaxsq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpmaxsq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Pmaxuw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaxuw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaxuw_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxuw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxuw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxuw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pmaxud_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaxud_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaxud_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmaxud_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpmaxud_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpmaxud_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpmaxuq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpmaxuq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpmaxuq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Pmulld_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmulld_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmulld_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmulld_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpmulld_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpmulld_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vpmullq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpmullq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpmullq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Phminposuw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vphminposuw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgetexpps_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vgetexpps_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vgetexpps_zmm_k1z_zmmm512b32_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vgetexppd_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vgetexppd_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vgetexppd_zmm_k1z_zmmm512b64_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vgetexpss_xmm_k1z_xmm_xmmm32_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgetexpsd_xmm_k1z_xmm_xmmm64_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vplzcntd_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vplzcntd_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vplzcntd_zmm_k1z_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vplzcntq_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vplzcntq_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vplzcntq_zmm_k1z_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.VEX_Vpsrlvd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrlvd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrlvq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsrlvq_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsrlvd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpsrlvd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpsrlvd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpsrlvq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpsrlvq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpsrlvq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.VEX_Vpsravd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsravd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsravd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpsravd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpsravd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpsravq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpsravq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpsravq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.VEX_Vpsllvd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsllvd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsllvq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpsllvq_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpsllvd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpsllvd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpsllvd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpsllvq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpsllvq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpsllvq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.EVEX_Vrcp14ps_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vrcp14ps_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vrcp14ps_zmm_k1z_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vrcp14pd_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vrcp14pd_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vrcp14pd_zmm_k1z_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vrcp14ss_xmm_k1z_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrcp14sd_xmm_k1z_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrsqrt14ps_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vrsqrt14ps_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vrsqrt14ps_zmm_k1z_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vrsqrt14pd_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vrsqrt14pd_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vrsqrt14pd_zmm_k1z_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vrsqrt14ss_xmm_k1z_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrsqrt14sd_xmm_k1z_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpdpbusd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpdpbusd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpdpbusd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpdpbusds_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpdpbusds_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpdpbusds_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpdpwssd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Broadcast128_2xInt16)]),
				(code[nameof(Code.EVEX_Vpdpwssd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Broadcast256_2xInt16)]),
				(code[nameof(Code.EVEX_Vpdpwssd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Broadcast512_2xInt16)]),
				(code[nameof(Code.EVEX_Vdpbf16ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_2xBFloat16)], memSize[nameof(MemorySize.Broadcast128_2xBFloat16)]),
				(code[nameof(Code.EVEX_Vdpbf16ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_2xBFloat16)], memSize[nameof(MemorySize.Broadcast256_2xBFloat16)]),
				(code[nameof(Code.EVEX_Vdpbf16ps_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_2xBFloat16)], memSize[nameof(MemorySize.Broadcast512_2xBFloat16)]),
				(code[nameof(Code.EVEX_Vp4dpwssd_zmm_k1z_zmmp3_m128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpdpwssds_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Broadcast128_2xInt16)]),
				(code[nameof(Code.EVEX_Vpdpwssds_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Broadcast256_2xInt16)]),
				(code[nameof(Code.EVEX_Vpdpwssds_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Broadcast512_2xInt16)]),
				(code[nameof(Code.EVEX_Vp4dpwssds_zmm_k1z_zmmp3_m128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpopcntb_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpopcntb_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpopcntb_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpopcntw_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpopcntw_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpopcntw_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpopcntd_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpopcntd_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpopcntd_zmm_k1z_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpopcntq_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpopcntq_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpopcntq_zmm_k1z_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.VEX_Vpbroadcastd_xmm_xmmm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpbroadcastd_ymm_xmmm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastd_xmm_k1z_xmmm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastd_ymm_k1z_xmmm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastd_zmm_k1z_xmmm32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpbroadcastq_xmm_xmmm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpbroadcastq_ymm_xmmm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcasti32x2_xmm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcasti32x2_ymm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcasti32x2_zmm_k1z_xmmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastq_xmm_k1z_xmmm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastq_ymm_k1z_xmmm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastq_zmm_k1z_xmmm64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vbroadcasti128_ymm_m128)], memSize[nameof(MemorySize.Int128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcasti32x4_ymm_k1z_m128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcasti32x4_zmm_k1z_m128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcasti64x2_ymm_k1z_m128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcasti64x2_zmm_k1z_m128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcasti32x8_zmm_k1z_m256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vbroadcasti64x4_zmm_k1z_m256)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpexpandb_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpexpandb_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpexpandb_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpexpandw_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpexpandw_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpexpandw_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcompressb_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcompressb_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcompressb_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcompressw_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcompressw_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcompressw_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpblendmd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpblendmd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpblendmd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vpblendmq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpblendmq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpblendmq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.EVEX_Vblendmps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vblendmps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vblendmps_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vblendmpd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vblendmpd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vblendmpd_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vpblendmb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpblendmb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpblendmb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpblendmw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpblendmw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpblendmw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vp2intersectd_kp1_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vp2intersectd_kp1_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vp2intersectd_kp1_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vp2intersectq_kp1_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vp2intersectq_kp1_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vp2intersectq_kp1_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.EVEX_Vpshldvw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshldvw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshldvw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshldvd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpshldvd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpshldvd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpshldvq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpshldvq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpshldvq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.EVEX_Vpshrdvw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshrdvw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshrdvw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtneps2bf16_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vcvtneps2bf16_xmm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vcvtneps2bf16_ymm_k1z_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vcvtne2ps2bf16_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vcvtne2ps2bf16_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vcvtne2ps2bf16_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vpshrdvd_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpshrdvd_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpshrdvd_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpshrdvq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpshrdvq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpshrdvq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.EVEX_Vpermi2b_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermi2b_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermi2b_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermi2w_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermi2w_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermi2w_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermi2d_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpermi2d_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpermi2d_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vpermi2q_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpermi2q_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpermi2q_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.EVEX_Vpermi2ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vpermi2ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vpermi2ps_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vpermi2pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vpermi2pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vpermi2pd_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vpbroadcastb_xmm_xmmm8)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpbroadcastb_ymm_xmmm8)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastb_xmm_k1z_xmmm8)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastb_ymm_k1z_xmmm8)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastb_zmm_k1z_xmmm8)], memSize[nameof(MemorySize.Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpbroadcastw_xmm_xmmm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpbroadcastw_ymm_xmmm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastw_xmm_k1z_xmmm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastw_ymm_k1z_xmmm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastw_zmm_k1z_xmmm16)], memSize[nameof(MemorySize.Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastb_xmm_k1z_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastb_ymm_k1z_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastb_zmm_k1z_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastw_xmm_k1z_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastw_ymm_k1z_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastw_zmm_k1z_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastd_xmm_k1z_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastd_ymm_k1z_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastd_zmm_k1z_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastq_xmm_k1z_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastq_ymm_k1z_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpbroadcastq_zmm_k1z_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermt2b_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermt2b_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermt2b_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermt2w_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermt2w_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermt2w_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermt2d_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpermt2d_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpermt2d_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vpermt2q_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpermt2q_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpermt2q_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.EVEX_Vpermt2ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vpermt2ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vpermt2ps_zmm_k1z_zmm_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vpermt2pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vpermt2pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vpermt2pd_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Invept_r32_m128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Invept_r64_m128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Invvpid_r32_m128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Invvpid_r64_m128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Invpcid_r32_m128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Invpcid_r64_m128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmultishiftqb_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpmultishiftqb_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpmultishiftqb_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.EVEX_Vexpandps_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vexpandps_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vexpandps_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vexpandpd_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vexpandpd_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vexpandpd_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpexpandd_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpexpandd_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpexpandd_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpexpandq_xmm_k1z_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpexpandq_ymm_k1z_ymmm256)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpexpandq_zmm_k1z_zmmm512)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcompressps_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcompressps_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcompressps_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcompresspd_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcompresspd_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcompresspd_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcompressd_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcompressd_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcompressd_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcompressq_xmmm128_k1z_xmm)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcompressq_ymmm256_k1z_ymm)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcompressq_zmmm512_k1z_zmm)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaskmovd_xmm_xmm_m128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaskmovd_ymm_ymm_m256)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaskmovq_xmm_xmm_m128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaskmovq_ymm_ymm_m256)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermw_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermw_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermw_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaskmovd_m128_xmm_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaskmovd_m256_ymm_ymm)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaskmovq_m128_xmm_xmm)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpmaskmovq_m256_ymm_ymm)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshufbitqmb_k_k1_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshufbitqmb_k_k1_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshufbitqmb_k_k1_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpgatherdd_xmm_vm32x_xmm)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpgatherdd_ymm_vm32y_ymm)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpgatherdq_xmm_vm32x_xmm)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpgatherdq_ymm_vm32x_ymm)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpgatherdd_xmm_k1_vm32x)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpgatherdd_ymm_k1_vm32y)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpgatherdd_zmm_k1_vm32z)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpgatherdq_xmm_k1_vm32x)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpgatherdq_ymm_k1_vm32x)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpgatherdq_zmm_k1_vm32y)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpgatherqd_xmm_vm64x_xmm)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpgatherqd_xmm_vm64y_xmm)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpgatherqq_xmm_vm64x_xmm)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpgatherqq_ymm_vm64y_ymm)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpgatherqd_xmm_k1_vm64x)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpgatherqd_xmm_k1_vm64y)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpgatherqd_ymm_k1_vm64z)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpgatherqq_xmm_k1_vm64x)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpgatherqq_ymm_k1_vm64y)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpgatherqq_zmm_k1_vm64z)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgatherdps_xmm_vm32x_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgatherdps_ymm_vm32y_ymm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgatherdpd_xmm_vm32x_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgatherdpd_ymm_vm32x_ymm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherdps_xmm_k1_vm32x)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherdps_ymm_k1_vm32y)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherdps_zmm_k1_vm32z)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherdpd_xmm_k1_vm32x)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherdpd_ymm_k1_vm32x)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherdpd_zmm_k1_vm32y)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgatherqps_xmm_vm64x_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgatherqps_xmm_vm64y_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgatherqpd_xmm_vm64x_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgatherqpd_ymm_vm64y_ymm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherqps_xmm_k1_vm64x)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherqps_xmm_k1_vm64y)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherqps_ymm_k1_vm64z)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherqpd_xmm_k1_vm64x)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherqpd_ymm_k1_vm64y)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherqpd_zmm_k1_vm64z)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsub132ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsub132ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsub132pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsub132pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmaddsub132ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfmaddsub132ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfmaddsub132ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfmaddsub132pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfmaddsub132pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfmaddsub132pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfmsubadd132ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubadd132ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubadd132pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubadd132pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmsubadd132ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfmsubadd132ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfmsubadd132ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfmsubadd132pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfmsubadd132pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfmsubadd132pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfmadd132ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmadd132ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmadd132pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmadd132pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmadd132ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfmadd132ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfmadd132ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfmadd132pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfmadd132pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfmadd132pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfmadd132ss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmadd132sd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmadd132ss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmadd132sd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub132ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub132ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub132pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub132pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmsub132ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfmsub132ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfmsub132ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfmsub132pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfmsub132pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfmsub132pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_V4fmaddps_zmm_k1z_zmmp3_m128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub132ss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub132sd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmsub132ss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmsub132sd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_V4fmaddss_xmm_k1z_xmmp3_m128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd132ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd132ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd132pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd132pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmadd132ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfnmadd132ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfnmadd132ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfnmadd132pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfnmadd132pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfnmadd132pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfnmadd132ss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd132sd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmadd132ss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmadd132sd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub132ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub132ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub132pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub132pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmsub132ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfnmsub132ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfnmsub132ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfnmsub132pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfnmsub132pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfnmsub132pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfnmsub132ss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub132sd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmsub132ss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmsub132sd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpscatterdd_vm32x_k1_xmm)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpscatterdd_vm32y_k1_ymm)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpscatterdd_vm32z_k1_zmm)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpscatterdq_vm32x_k1_xmm)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpscatterdq_vm32x_k1_ymm)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpscatterdq_vm32y_k1_zmm)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpscatterqd_vm64x_k1_xmm)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpscatterqd_vm64y_k1_xmm)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpscatterqd_vm64z_k1_ymm)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpscatterqq_vm64x_k1_xmm)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpscatterqq_vm64y_k1_ymm)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpscatterqq_vm64z_k1_zmm)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterdps_vm32x_k1_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterdps_vm32y_k1_ymm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterdps_vm32z_k1_zmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterdpd_vm32x_k1_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterdpd_vm32x_k1_ymm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterdpd_vm32y_k1_zmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterqps_vm64x_k1_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterqps_vm64y_k1_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterqps_vm64z_k1_ymm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterqpd_vm64x_k1_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterqpd_vm64y_k1_ymm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterqpd_vm64z_k1_zmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsub213ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsub213ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsub213pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsub213pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmaddsub213ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfmaddsub213ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfmaddsub213ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfmaddsub213pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfmaddsub213pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfmaddsub213pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfmsubadd213ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubadd213ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubadd213pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubadd213pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmsubadd213ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfmsubadd213ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfmsubadd213ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfmsubadd213pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfmsubadd213pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfmsubadd213pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfmadd213ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmadd213ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmadd213pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmadd213pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmadd213ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfmadd213ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfmadd213ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfmadd213pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfmadd213pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfmadd213pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfmadd213ss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmadd213sd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmadd213ss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmadd213sd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub213ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub213ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub213pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub213pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmsub213ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfmsub213ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfmsub213ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfmsub213pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfmsub213pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfmsub213pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_V4fnmaddps_zmm_k1z_zmmp3_m128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub213ss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub213sd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmsub213ss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmsub213sd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_V4fnmaddss_xmm_k1z_xmmp3_m128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd213ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd213ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd213pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd213pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmadd213ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfnmadd213ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfnmadd213ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfnmadd213pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfnmadd213pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfnmadd213pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfnmadd213ss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd213sd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmadd213ss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmadd213sd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub213ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub213ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub213pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub213pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmsub213ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfnmsub213ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfnmsub213pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfnmsub213pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfnmsub213pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfnmsub213ss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub213sd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmsub213ss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmsub213sd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpmadd52luq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt52)], memSize[nameof(MemorySize.Broadcast128_UInt52)]),
				(code[nameof(Code.EVEX_Vpmadd52luq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt52)], memSize[nameof(MemorySize.Broadcast256_UInt52)]),
				(code[nameof(Code.EVEX_Vpmadd52luq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt52)], memSize[nameof(MemorySize.Broadcast512_UInt52)]),
				(code[nameof(Code.EVEX_Vpmadd52huq_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_UInt52)], memSize[nameof(MemorySize.Broadcast128_UInt52)]),
				(code[nameof(Code.EVEX_Vpmadd52huq_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_UInt52)], memSize[nameof(MemorySize.Broadcast256_UInt52)]),
				(code[nameof(Code.EVEX_Vpmadd52huq_zmm_k1z_zmm_zmmm512b64)], memSize[nameof(MemorySize.Packed512_UInt52)], memSize[nameof(MemorySize.Broadcast512_UInt52)]),
				(code[nameof(Code.VEX_Vfmaddsub231ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsub231ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsub231pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsub231pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmaddsub231ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfmaddsub231ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfmaddsub231ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfmaddsub231pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfmaddsub231pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfmaddsub231pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfmsubadd231ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubadd231ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubadd231pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubadd231pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmsubadd231ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfmsubadd231ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfmsubadd231ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfmsubadd231pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfmsubadd231pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfmsubadd231pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfmadd231ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmadd231ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmadd231pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmadd231pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmadd231ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfmadd231ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfmadd231ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfmadd231pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfmadd231pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfmadd231pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfmadd231ss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmadd231sd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmadd231ss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmadd231sd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub231ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub231ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub231pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub231pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmsub231ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfmsub231ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfmsub231ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfmsub231pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfmsub231pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfmsub231pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfmsub231ss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsub231sd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmsub231ss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfmsub231sd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd231ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd231ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd231pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd231pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmadd231ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfnmadd231ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfnmadd231pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfnmadd231pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfnmadd231pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfnmadd231ss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmadd231sd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmadd231ss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmadd231sd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub231ps_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub231ps_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub231pd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub231pd_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmsub231ps_xmm_k1z_xmm_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfnmsub231ps_ymm_k1z_ymm_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfnmsub231ps_zmm_k1z_zmm_zmmm512b32_er)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfnmsub231pd_xmm_k1z_xmm_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfnmsub231pd_ymm_k1z_ymm_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfnmsub231pd_zmm_k1z_zmm_zmmm512b64_er)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vfnmsub231ss_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsub231sd_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmsub231ss_xmm_k1z_xmm_xmmm32_er)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfnmsub231sd_xmm_k1z_xmm_xmmm64_er)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpconflictd_xmm_k1z_xmmm128b32)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpconflictd_ymm_k1z_ymmm256b32)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpconflictd_zmm_k1z_zmmm512b32)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vpconflictq_xmm_k1z_xmmm128b64)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpconflictq_ymm_k1z_ymmm256b64)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpconflictq_zmm_k1z_zmmm512b64)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.EVEX_Vgatherpf0dps_vm32z_k1)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherpf0dpd_vm32y_k1)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherpf1dps_vm32z_k1)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherpf1dpd_vm32y_k1)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterpf0dps_vm32z_k1)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterpf0dpd_vm32y_k1)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterpf1dps_vm32z_k1)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterpf1dpd_vm32y_k1)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherpf0qps_vm64z_k1)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherpf0qpd_vm64z_k1)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherpf1qps_vm64z_k1)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgatherpf1qpd_vm64z_k1)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterpf0qps_vm64z_k1)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterpf0qpd_vm64z_k1)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterpf1qps_vm64z_k1)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vscatterpf1qpd_vm64z_k1)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sha1nexte_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vexp2ps_zmm_k1z_zmmm512b32_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vexp2pd_zmm_k1z_zmmm512b64_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Sha1msg1_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sha1msg2_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrcp28ps_zmm_k1z_zmmm512b32_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vrcp28pd_zmm_k1z_zmmm512b64_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Sha256rnds2_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrcp28ss_xmm_k1z_xmm_xmmm32_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrcp28sd_xmm_k1z_xmm_xmmm64_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sha256msg1_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrsqrt28ps_zmm_k1z_zmmm512b32_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vrsqrt28pd_zmm_k1z_zmmm512b64_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Sha256msg2_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrsqrt28ss_xmm_k1z_xmm_xmmm32_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrsqrt28sd_xmm_k1z_xmm_xmmm64_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Gf2p8mulb_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgf2p8mulb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgf2p8mulb_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgf2p8mulb_xmm_k1z_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgf2p8mulb_ymm_k1z_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgf2p8mulb_zmm_k1z_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Aesimc_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaesimc_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Aesenc_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaesenc_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaesenc_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaesenc_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaesenc_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaesenc_zmm_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Aesenclast_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaesenclast_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaesenclast_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaesenclast_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaesenclast_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaesenclast_zmm_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Aesdec_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaesdec_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaesdec_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaesdec_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaesdec_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaesdec_zmm_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Aesdeclast_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaesdeclast_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaesdeclast_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaesdeclast_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaesdeclast_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vaesdeclast_zmm_zmm_zmmm512)], memSize[nameof(MemorySize.Packed512_UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movbe_r16_m16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movbe_r32_m32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movbe_r64_m64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Crc32_r32_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Crc32_r64_rm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movbe_m16_r16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movbe_m32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movbe_m64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Crc32_r32_rm16)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Crc32_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Crc32_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Andn_r32_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Andn_r64_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Blsr_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Blsr_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Blsmsk_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Blsmsk_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Blsi_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Blsi_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Bzhi_r32_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Bzhi_r64_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wrussd_m32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wrussq_m64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Pext_r32_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Pext_r64_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Pdep_r32_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Pdep_r64_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wrssd_m32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Wrssq_m64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adcx_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adcx_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adox_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Adox_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Mulx_r32_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Mulx_r64_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Bextr_r32_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Bextr_r64_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Shlx_r32_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Shlx_r64_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Sarx_r32_rm32_r32)], memSize[nameof(MemorySize.Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Sarx_r64_rm64_r64)], memSize[nameof(MemorySize.Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Shrx_r32_rm32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Shrx_r64_rm64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movdir64b_r16_m512)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movdir64b_r32_m512)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movdir64b_r64_m512)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Enqcmds_r16_m512)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Enqcmds_r32_m512)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Enqcmds_r64_m512)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Enqcmd_r16_m512)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Enqcmd_r32_m512)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Enqcmd_r64_m512)], memSize[nameof(MemorySize.UInt512)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movdiri_m32_r32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Movdiri_m64_r64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermq_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermq_ymm_k1z_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpermq_zmm_k1z_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.VEX_Vpermpd_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermpd_ymm_k1z_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vpermpd_zmm_k1z_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vpblendd_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpblendd_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Valignd_xmm_k1z_xmm_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Valignd_ymm_k1z_ymm_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Valignd_zmm_k1z_zmm_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Valignq_xmm_k1z_xmm_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Valignq_ymm_k1z_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Valignq_zmm_k1z_zmm_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.VEX_Vpermilps_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermilps_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermilps_xmm_k1z_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vpermilps_ymm_k1z_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vpermilps_zmm_k1z_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.VEX_Vpermilpd_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermilpd_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpermilpd_xmm_k1z_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vpermilpd_ymm_k1z_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vpermilpd_zmm_k1z_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.VEX_Vperm2f128_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Roundps_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vroundps_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vroundps_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrndscaleps_xmm_k1z_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vrndscaleps_ymm_k1z_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vrndscaleps_zmm_k1z_zmmm512b32_imm8_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.Roundpd_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vroundpd_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vroundpd_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrndscalepd_xmm_k1z_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vrndscalepd_ymm_k1z_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vrndscalepd_zmm_k1z_zmmm512b64_imm8_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.Roundss_xmm_xmmm32_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vroundss_xmm_xmm_xmmm32_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrndscaless_xmm_k1z_xmm_xmmm32_imm8_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Roundsd_xmm_xmmm64_imm8)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vroundsd_xmm_xmm_xmmm64_imm8)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrndscalesd_xmm_k1z_xmm_xmmm64_imm8_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Blendps_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vblendps_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vblendps_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Blendpd_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vblendpd_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vblendpd_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pblendw_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpblendw_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpblendw_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Palignr_mm_mmm64_imm8)], memSize[nameof(MemorySize.Packed64_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Palignr_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpalignr_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpalignr_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpalignr_xmm_k1z_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpalignr_ymm_k1z_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpalignr_zmm_k1z_zmm_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pextrb_r32m8_xmm_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pextrb_r64m8_xmm_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpextrb_r32m8_xmm_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpextrb_r64m8_xmm_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpextrb_r32m8_xmm_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpextrb_r64m8_xmm_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pextrw_r32m16_xmm_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pextrw_r64m16_xmm_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpextrw_r32m16_xmm_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpextrw_r64m16_xmm_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpextrw_r32m16_xmm_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpextrw_r64m16_xmm_imm8)], memSize[nameof(MemorySize.UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pextrd_rm32_xmm_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pextrq_rm64_xmm_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpextrd_rm32_xmm_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpextrq_rm64_xmm_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpextrd_rm32_xmm_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpextrq_rm64_xmm_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Extractps_rm32_xmm_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Extractps_r64m32_xmm_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vextractps_rm32_xmm_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vextractps_r64m32_xmm_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextractps_rm32_xmm_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextractps_r64m32_xmm_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vinsertf128_ymm_ymm_xmmm128_imm8)], memSize[nameof(MemorySize.Float128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinsertf32x4_ymm_k1z_ymm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinsertf32x4_zmm_k1z_zmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinsertf64x2_ymm_k1z_ymm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinsertf64x2_zmm_k1z_zmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vextractf128_xmmm128_ymm_imm8)], memSize[nameof(MemorySize.Float128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextractf32x4_xmmm128_k1z_ymm_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextractf32x4_xmmm128_k1z_zmm_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextractf64x2_xmmm128_k1z_ymm_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextractf64x2_xmmm128_k1z_zmm_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinsertf32x8_zmm_k1z_zmm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinsertf64x4_zmm_k1z_zmm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextractf32x8_ymmm256_k1z_zmm_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextractf64x4_ymmm256_k1z_zmm_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtps2ph_xmmm64_xmm_imm8)], memSize[nameof(MemorySize.Packed64_Float16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vcvtps2ph_xmmm128_ymm_imm8)], memSize[nameof(MemorySize.Packed128_Float16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtps2ph_xmmm64_k1z_xmm_imm8)], memSize[nameof(MemorySize.Packed64_Float16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtps2ph_xmmm128_k1z_ymm_imm8)], memSize[nameof(MemorySize.Packed128_Float16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vcvtps2ph_ymmm256_k1z_zmm_imm8_sae)], memSize[nameof(MemorySize.Packed256_Float16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpud_k_k1_xmm_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpcmpud_k_k1_ymm_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpcmpud_k_k1_zmm_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpcmpuq_k_k1_xmm_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpcmpuq_k_k1_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpcmpuq_k_k1_zmm_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.EVEX_Vpcmpd_k_k1_xmm_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpcmpd_k_k1_ymm_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpcmpd_k_k1_zmm_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vpcmpq_k_k1_xmm_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpcmpq_k_k1_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpcmpq_k_k1_zmm_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Pinsrb_xmm_r32m8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pinsrb_xmm_r64m8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpinsrb_xmm_xmm_r32m8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpinsrb_xmm_xmm_r64m8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpinsrb_xmm_xmm_r32m8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpinsrb_xmm_xmm_r64m8_imm8)], memSize[nameof(MemorySize.UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Insertps_xmm_xmmm32_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vinsertps_xmm_xmm_xmmm32_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinsertps_xmm_xmm_xmmm32_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pinsrd_xmm_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pinsrq_xmm_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpinsrd_xmm_xmm_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpinsrq_xmm_xmm_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpinsrd_xmm_xmm_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpinsrq_xmm_xmm_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vshuff32x4_ymm_k1z_ymm_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vshuff32x4_zmm_k1z_zmm_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vshuff64x2_ymm_k1z_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vshuff64x2_zmm_k1z_zmm_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vpternlogd_xmm_k1z_xmm_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Broadcast128_Int32)]),
				(code[nameof(Code.EVEX_Vpternlogd_ymm_k1z_ymm_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vpternlogd_zmm_k1z_zmm_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vpternlogq_xmm_k1z_xmm_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Broadcast128_Int64)]),
				(code[nameof(Code.EVEX_Vpternlogq_ymm_k1z_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vpternlogq_zmm_k1z_zmm_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.EVEX_Vgetmantps_xmm_k1z_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vgetmantps_ymm_k1z_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vgetmantps_zmm_k1z_zmmm512b32_imm8_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vgetmantpd_xmm_k1z_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vgetmantpd_ymm_k1z_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vgetmantpd_zmm_k1z_zmmm512b64_imm8_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vgetmantss_xmm_k1z_xmm_xmmm32_imm8_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgetmantsd_xmm_k1z_xmm_xmmm64_imm8_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kshiftrb_k_k_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kshiftrw_k_k_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kshiftrd_k_k_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kshiftrq_k_k_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kshiftlb_k_k_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kshiftlw_k_k_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kshiftld_k_k_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Kshiftlq_k_k_imm8)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vinserti128_ymm_ymm_xmmm128_imm8)], memSize[nameof(MemorySize.Int128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinserti32x4_ymm_k1z_ymm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinserti32x4_zmm_k1z_zmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinserti64x2_ymm_k1z_ymm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinserti64x2_zmm_k1z_zmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vextracti128_xmmm128_ymm_imm8)], memSize[nameof(MemorySize.Int128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextracti32x4_xmmm128_k1z_ymm_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextracti32x4_xmmm128_k1z_zmm_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextracti64x2_xmmm128_k1z_ymm_imm8)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextracti64x2_xmmm128_k1z_zmm_imm8)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinserti32x8_zmm_k1z_zmm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vinserti64x4_zmm_k1z_zmm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextracti32x8_ymmm256_k1z_zmm_imm8)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vextracti64x4_ymmm256_k1z_zmm_imm8)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpub_k_k1_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpub_k_k1_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpub_k_k1_zmm_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpuw_k_k1_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpuw_k_k1_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpuw_k_k1_zmm_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpb_k_k1_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpb_k_k1_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpb_k_k1_zmm_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpw_k_k1_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpw_k_k1_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpcmpw_k_k1_zmm_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Dpps_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vdpps_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vdpps_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Dppd_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vdppd_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Mpsadbw_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmpsadbw_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vmpsadbw_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vdbpsadbw_xmm_k1z_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vdbpsadbw_ymm_k1z_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vdbpsadbw_zmm_k1z_zmm_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vshufi32x4_ymm_k1z_ymm_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Int32)], memSize[nameof(MemorySize.Broadcast256_Int32)]),
				(code[nameof(Code.EVEX_Vshufi32x4_zmm_k1z_zmm_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_Int32)], memSize[nameof(MemorySize.Broadcast512_Int32)]),
				(code[nameof(Code.EVEX_Vshufi64x2_ymm_k1z_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Int64)], memSize[nameof(MemorySize.Broadcast256_Int64)]),
				(code[nameof(Code.EVEX_Vshufi64x2_zmm_k1z_zmm_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_Int64)], memSize[nameof(MemorySize.Broadcast512_Int64)]),
				(code[nameof(Code.Pclmulqdq_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vperm2i128_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_Int128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermil2ps_xmm_xmm_xmmm128_xmm_imm2)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermil2ps_ymm_ymm_ymmm256_ymm_imm2)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermil2ps_xmm_xmm_xmm_xmmm128_imm2)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermil2ps_ymm_ymm_ymm_ymmm256_imm2)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermil2pd_xmm_xmm_xmmm128_xmm_imm2)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermil2pd_ymm_ymm_ymmm256_ymm_imm2)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermil2pd_xmm_xmm_xmm_xmmm128_imm2)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpermil2pd_ymm_ymm_ymm_ymmm256_imm2)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vblendvps_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vblendvps_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vblendvpd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vblendvpd_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpblendvb_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpblendvb_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrangeps_xmm_k1z_xmm_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vrangeps_ymm_k1z_ymm_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vrangeps_zmm_k1z_zmm_zmmm512b32_imm8_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vrangepd_xmm_k1z_xmm_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vrangepd_ymm_k1z_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vrangepd_zmm_k1z_zmm_zmmm512b64_imm8_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vrangess_xmm_k1z_xmm_xmmm32_imm8_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vrangesd_xmm_k1z_xmm_xmmm64_imm8_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfixupimmps_xmm_k1z_xmm_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfixupimmps_ymm_k1z_ymm_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfixupimmps_zmm_k1z_zmm_zmmm512b32_imm8_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfixupimmpd_xmm_k1z_xmm_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfixupimmpd_ymm_k1z_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vfixupimmss_xmm_k1z_xmm_xmmm32_imm8_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfixupimmsd_xmm_k1z_xmm_xmmm64_imm8_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vreduceps_xmm_k1z_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vreduceps_ymm_k1z_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vreduceps_zmm_k1z_zmmm512b32_imm8_sae)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vreducepd_xmm_k1z_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vreducepd_ymm_k1z_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vreducepd_zmm_k1z_zmmm512b64_imm8_sae)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vreducess_xmm_k1z_xmm_xmmm32_imm8_sae)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vreducesd_xmm_k1z_xmm_xmmm64_imm8_sae)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsubps_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsubps_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsubps_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsubps_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsubpd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsubpd_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsubpd_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsubpd_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubaddps_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubaddps_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubaddps_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubaddps_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubaddpd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubaddpd_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubaddpd_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubaddpd_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpestrm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpestrm64_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpestrm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpestrm64_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpestri_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpestri64_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpestri_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpestri64_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpistrm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpistrm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Pcmpistri_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vpcmpistri_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfpclassps_k_k1_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Broadcast128_Float32)]),
				(code[nameof(Code.EVEX_Vfpclassps_k_k1_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Broadcast256_Float32)]),
				(code[nameof(Code.EVEX_Vfpclassps_k_k1_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_Float32)], memSize[nameof(MemorySize.Broadcast512_Float32)]),
				(code[nameof(Code.EVEX_Vfpclasspd_k_k1_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Broadcast128_Float64)]),
				(code[nameof(Code.EVEX_Vfpclasspd_k_k1_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Broadcast256_Float64)]),
				(code[nameof(Code.EVEX_Vfpclasspd_k_k1_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_Float64)], memSize[nameof(MemorySize.Broadcast512_Float64)]),
				(code[nameof(Code.EVEX_Vfpclassss_k_k1_xmmm32_imm8)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vfpclasssd_k_k1_xmmm64_imm8)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddps_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddps_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddps_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddps_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddpd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddpd_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddpd_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddpd_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddss_xmm_xmm_xmmm32_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddss_xmm_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsd_xmm_xmm_xmmm64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmaddsd_xmm_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubps_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubps_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubps_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubps_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubpd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubpd_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubpd_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubpd_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubss_xmm_xmm_xmmm32_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubss_xmm_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubsd_xmm_xmm_xmmm64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfmsubsd_xmm_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshldw_xmm_k1z_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshldw_ymm_k1z_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshldw_zmm_k1z_zmm_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshldd_xmm_k1z_xmm_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpshldd_ymm_k1z_ymm_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpshldd_zmm_k1z_zmm_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpshldq_xmm_k1z_xmm_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpshldq_ymm_k1z_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpshldq_zmm_k1z_zmm_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.EVEX_Vpshrdw_xmm_k1z_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshrdw_ymm_k1z_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshrdw_zmm_k1z_zmm_zmmm512_imm8)], memSize[nameof(MemorySize.Packed512_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vpshrdd_xmm_k1z_xmm_xmmm128b32_imm8)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Broadcast128_UInt32)]),
				(code[nameof(Code.EVEX_Vpshrdd_ymm_k1z_ymm_ymmm256b32_imm8)], memSize[nameof(MemorySize.Packed256_UInt32)], memSize[nameof(MemorySize.Broadcast256_UInt32)]),
				(code[nameof(Code.EVEX_Vpshrdd_zmm_k1z_zmm_zmmm512b32_imm8)], memSize[nameof(MemorySize.Packed512_UInt32)], memSize[nameof(MemorySize.Broadcast512_UInt32)]),
				(code[nameof(Code.EVEX_Vpshrdq_xmm_k1z_xmm_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vpshrdq_ymm_k1z_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_UInt64)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vpshrdq_zmm_k1z_zmm_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_UInt64)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.VEX_Vfnmaddps_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmaddps_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmaddps_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmaddps_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmaddpd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmaddpd_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmaddpd_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmaddpd_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmaddss_xmm_xmm_xmmm32_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmaddss_xmm_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmaddsd_xmm_xmm_xmmm64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmaddsd_xmm_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsubps_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsubps_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsubps_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsubps_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsubpd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsubpd_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsubpd_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsubpd_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsubss_xmm_xmm_xmmm32_xmm)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsubss_xmm_xmm_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsubsd_xmm_xmm_xmmm64_xmm)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vfnmsubsd_xmm_xmm_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Sha1rnds4_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.Gf2p8affineqb_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgf2p8affineqb_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgf2p8affineqb_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgf2p8affineqb_xmm_k1z_xmm_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vgf2p8affineqb_ymm_k1z_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vgf2p8affineqb_zmm_k1z_zmm_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Gf2p8affineinvqb_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgf2p8affineinvqb_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vgf2p8affineinvqb_ymm_ymm_ymmm256_imm8)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.EVEX_Vgf2p8affineinvqb_xmm_k1z_xmm_xmmm128b64_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Broadcast128_UInt64)]),
				(code[nameof(Code.EVEX_Vgf2p8affineinvqb_ymm_k1z_ymm_ymmm256b64_imm8)], memSize[nameof(MemorySize.Packed256_UInt8)], memSize[nameof(MemorySize.Broadcast256_UInt64)]),
				(code[nameof(Code.EVEX_Vgf2p8affineinvqb_zmm_k1z_zmm_zmmm512b64_imm8)], memSize[nameof(MemorySize.Packed512_UInt8)], memSize[nameof(MemorySize.Broadcast512_UInt64)]),
				(code[nameof(Code.Aeskeygenassist_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Vaeskeygenassist_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Rorx_r32_rm32_imm8)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.VEX_Rorx_r64_rm64_imm8)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpmacssww_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpmacsswd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpmacssdql_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpmacssdd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpmacssdqh_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpmacsww_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpmacswd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpmacsdql_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpmacsdd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpmacsdqh_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpcmov_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpcmov_ymm_ymm_ymmm256_ymm)], memSize[nameof(MemorySize.UInt256)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpcmov_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.UInt128)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpcmov_ymm_ymm_ymm_ymmm256)], memSize[nameof(MemorySize.UInt256)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpperm_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpperm_xmm_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpmadcsswd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpmadcswd_xmm_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vprotb_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vprotw_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vprotd_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vprotq_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blcfill_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blcfill_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blsfill_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blsfill_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blcs_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blcs_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Tzmsk_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Tzmsk_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blcic_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blcic_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blsic_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blsic_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_T1mskc_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_T1mskc_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blcmsk_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blcmsk_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blci_r32_rm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Blci_r64_rm64)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Llwpcb_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Llwpcb_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Slwpcb_r32)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Slwpcb_r64)], memSize[nameof(MemorySize.Unknown)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vfrczps_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vfrczps_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vfrczpd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vfrczpd_ymm_ymmm256)], memSize[nameof(MemorySize.Packed256_Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vfrczss_xmm_xmmm32)], memSize[nameof(MemorySize.Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vfrczsd_xmm_xmmm64)], memSize[nameof(MemorySize.Float64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vprotb_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vprotb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vprotw_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vprotw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vprotd_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vprotd_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vprotq_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vprotq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshlb_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshlb_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshlw_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshlw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshld_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshld_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshlq_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshlq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshab_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshab_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshaw_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshaw_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshad_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshad_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshaq_xmm_xmmm128_xmm)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vpshaq_xmm_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphaddbw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphaddbd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphaddbq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphaddwd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphaddwq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphadddq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphaddubw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphaddubd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphaddubq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphadduwd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphadduwq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphaddudq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphsubbw_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int8)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphsubwd_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Vphsubdq_xmm_xmmm128)], memSize[nameof(MemorySize.Packed128_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Bextr_r32_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Bextr_r64_rm64_imm32)], memSize[nameof(MemorySize.UInt64)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Lwpins_r32_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Lwpins_r64_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Lwpval_r32_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.XOP_Lwpval_r64_rm32_imm32)], memSize[nameof(MemorySize.UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pi2fw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pi2fd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pf2iw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pf2id_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfrcpv_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfrsqrtv_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfnacc_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfpnacc_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfcmpge_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfmin_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfrcp_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfrsqrt_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfsub_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfadd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfcmpgt_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfmax_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfrcpit1_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfrsqit1_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfsubr_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfacc_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfcmpeq_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfmul_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pfrcpit2_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Float32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pmulhrw_mm_mmm64)], memSize[nameof(MemorySize.Packed64_Int16)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pswapd_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt32)], memSize[nameof(MemorySize.Unknown)]),
				(code[nameof(Code.D3NOW_Pavgusb_mm_mmm64)], memSize[nameof(MemorySize.Packed64_UInt8)], memSize[nameof(MemorySize.Unknown)]),
			};
			if (result.Length != code.Values.Length)
				throw new InvalidOperationException();
			if (result.Select(a => a.codeEnum).ToHashSet<EnumValue>().Count != code.Values.Length)
				throw new InvalidOperationException();
			Array.Sort(result, (a, b) => a.codeEnum.Value.CompareTo(b.codeEnum.Value));
			return result;
		}
	}
}
