/*
Copyright (C) 2018-2019 de4dot@gmail.com

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
"Software"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be
included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#if (!NO_GAS_FORMATTER || !NO_INTEL_FORMATTER || !NO_MASM_FORMATTER || !NO_NASM_FORMATTER) && !NO_FORMATTER
using Iced.Intel;

namespace Iced.UnitTests.Intel.FormatterTests {
	static class FormatterOptionsTests {
		public static readonly OptionsInstructionInfo[] AllInfos = new OptionsInstructionInfo[] {
			new OptionsInstructionInfo(64, "64 F3 A4", Code.Movsb_m8_m8, a => a.UpperCasePrefixes = true),
			new OptionsInstructionInfo(64, "64 F3 A4", Code.Movsb_m8_m8, a => a.UpperCasePrefixes = false),

			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => a.UpperCaseMnemonics = true),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => a.UpperCaseMnemonics = false),

			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => a.UpperCaseRegisters = true),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => a.UpperCaseRegisters = false),

			new OptionsInstructionInfo(64, "70 00", Code.Jo_rel8_64, a => { a.UpperCaseKeywords = true; a.ShowBranchSize = true; }),
			new OptionsInstructionInfo(64, "70 00", Code.Jo_rel8_64, a => { a.UpperCaseKeywords = false; a.ShowBranchSize = true; }),

			new OptionsInstructionInfo(64, "62 F14E9B 51 D3", Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er, a => a.UpperCaseDecorators = true),
			new OptionsInstructionInfo(64, "62 F14E9B 51 D3", Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er, a => a.UpperCaseDecorators = false),
			new OptionsInstructionInfo(64, "62 F14EBB 51 D3", Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er, a => a.UpperCaseDecorators = true),
			new OptionsInstructionInfo(64, "62 F14EBB 51 D3", Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er, a => a.UpperCaseDecorators = false),
			new OptionsInstructionInfo(64, "62 F14EDB 51 D3", Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er, a => a.UpperCaseDecorators = true),
			new OptionsInstructionInfo(64, "62 F14EDB 51 D3", Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er, a => a.UpperCaseDecorators = false),
			new OptionsInstructionInfo(64, "62 F14EFB 51 D3", Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er, a => a.UpperCaseDecorators = true),
			new OptionsInstructionInfo(64, "62 F14EFB 51 D3", Code.EVEX_Vsqrtss_xmm_k1z_xmm_xmmm32_er, a => a.UpperCaseDecorators = false),
			new OptionsInstructionInfo(64, "62 F3CDDB 54 D3 A5", Code.EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae, a => a.UpperCaseDecorators = true),
			new OptionsInstructionInfo(64, "62 F3CDDB 54 D3 A5", Code.EVEX_Vfixupimmpd_zmm_k1z_zmm_zmmm512b64_imm8_sae, a => a.UpperCaseDecorators = false),

			new OptionsInstructionInfo(64, "64 F3 A4", Code.Movsb_m8_m8, a => { a.UpperCasePrefixes = false; a.UpperCaseMnemonics = false; a.UpperCaseRegisters = false; a.UpperCaseKeywords = false; a.UpperCaseDecorators = false; a.UpperCaseAll = true; }),
			new OptionsInstructionInfo(64, "64 F3 A4", Code.Movsb_m8_m8, a => { a.UpperCasePrefixes = false; a.UpperCaseMnemonics = false; a.UpperCaseRegisters = false; a.UpperCaseKeywords = false; a.UpperCaseDecorators = false; a.UpperCaseAll = false; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.UpperCasePrefixes = false; a.UpperCaseMnemonics = false; a.UpperCaseRegisters = false; a.UpperCaseKeywords = false; a.UpperCaseDecorators = false; a.UpperCaseAll = true; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.UpperCasePrefixes = false; a.UpperCaseMnemonics = false; a.UpperCaseRegisters = false; a.UpperCaseKeywords = false; a.UpperCaseDecorators = false; a.UpperCaseAll = false; }),
			new OptionsInstructionInfo(64, "62 F24DDB AE D3", Code.EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.UpperCasePrefixes = false; a.UpperCaseMnemonics = false; a.UpperCaseRegisters = false; a.UpperCaseKeywords = false; a.UpperCaseDecorators = false; a.UpperCaseAll = true; }),
			new OptionsInstructionInfo(64, "62 F24DDB AE D3", Code.EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.UpperCasePrefixes = false; a.UpperCaseMnemonics = false; a.UpperCaseRegisters = false; a.UpperCaseKeywords = false; a.UpperCaseDecorators = false; a.UpperCaseAll = false; }),
			new OptionsInstructionInfo(64, "70 00", Code.Jo_rel8_64, a => { a.ShowBranchSize = true; a.UpperCasePrefixes = false; a.UpperCaseMnemonics = false; a.UpperCaseRegisters = false; a.UpperCaseKeywords = false; a.UpperCaseDecorators = false; a.UpperCaseAll = true; }),
			new OptionsInstructionInfo(64, "70 00", Code.Jo_rel8_64, a => { a.ShowBranchSize = true; a.UpperCasePrefixes = false; a.UpperCaseMnemonics = false; a.UpperCaseRegisters = false; a.UpperCaseKeywords = false; a.UpperCaseDecorators = false; a.UpperCaseAll = false; }),

			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -1; a.FirstOperandCharIndex = -1; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = -1; }),

			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -9; a.FirstOperandCharIndex = 0; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -8; a.FirstOperandCharIndex = 1; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -7; a.FirstOperandCharIndex = 2; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -6; a.FirstOperandCharIndex = 3; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -5; a.FirstOperandCharIndex = 4; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -4; a.FirstOperandCharIndex = 5; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -3; a.FirstOperandCharIndex = 6; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -2; a.FirstOperandCharIndex = 7; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -1; a.FirstOperandCharIndex = 8; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 0; a.FirstOperandCharIndex = 9; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -9; a.FirstOperandCharIndex = 10; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -8; a.FirstOperandCharIndex = 11; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -7; a.FirstOperandCharIndex = 12; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -6; a.FirstOperandCharIndex = 13; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -5; a.FirstOperandCharIndex = 14; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -4; a.FirstOperandCharIndex = 15; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -3; a.FirstOperandCharIndex = 16; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -2; a.FirstOperandCharIndex = 17; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -1; a.FirstOperandCharIndex = 18; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 0; a.FirstOperandCharIndex = 19; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = -9; a.FirstOperandCharIndex = 20; }),

			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 0; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 1; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 2; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 3; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 4; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 5; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 6; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 7; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 8; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 9; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 10; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 11; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 12; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 13; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 14; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 15; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 16; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 17; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 18; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 19; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 1; a.FirstOperandCharIndex = 20; }),

			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 0; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 1; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 2; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 3; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 4; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 5; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 6; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 7; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 8; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 9; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 10; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 11; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 12; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 13; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 14; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 15; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 16; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 17; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 18; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 19; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 2; a.FirstOperandCharIndex = 20; }),

			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 0; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 1; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 2; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 3; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 4; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 5; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 6; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 7; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 8; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 9; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 10; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 11; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 12; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 13; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 14; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 15; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 16; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 17; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 18; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 19; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 3; a.FirstOperandCharIndex = 20; }),

			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 0; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 1; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 2; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 3; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 4; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 5; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 6; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 7; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 8; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 9; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 10; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 11; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 12; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 13; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 14; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 15; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 16; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 17; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 18; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 19; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 4; a.FirstOperandCharIndex = 20; }),

			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 0; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 1; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 2; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 3; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 4; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 5; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 6; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 7; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 8; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 9; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 10; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 11; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 12; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 13; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 14; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 15; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 16; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 17; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 18; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 19; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 8; a.FirstOperandCharIndex = 20; }),

			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 0; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 1; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 2; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 3; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 4; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 5; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 6; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 7; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 8; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 9; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 10; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 11; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 12; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 13; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 14; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 15; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 16; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 17; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 18; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 19; }),
			new OptionsInstructionInfo(64, "0F 20 E1", Code.Mov_r64_cr, a => { a.TabSize = 100; a.FirstOperandCharIndex = 20; }),

			new OptionsInstructionInfo(64, "62 F24DDB AE D3", Code.EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er, a => a.SpaceAfterOperandSeparator = true),
			new OptionsInstructionInfo(64, "62 F24DDB AE D3", Code.EVEX_Vfnmsub213ps_zmm_k1z_zmm_zmmm512b32_er, a => a.SpaceAfterOperandSeparator = false),

			new OptionsInstructionInfo(64, "C7 44 48 EE 88A9CBED", Code.Mov_rm32_imm32, a => a.SpaceAfterMemoryBracket = true),
			new OptionsInstructionInfo(64, "C7 44 48 EE 88A9CBED", Code.Mov_rm32_imm32, a => a.SpaceAfterMemoryBracket = false),

			new OptionsInstructionInfo(64, "C7 44 48 EE 88A9CBED", Code.Mov_rm32_imm32, a => a.SpaceBetweenMemoryAddOperators = true),
			new OptionsInstructionInfo(64, "C7 44 48 EE 88A9CBED", Code.Mov_rm32_imm32, a => a.SpaceBetweenMemoryAddOperators = false),

			new OptionsInstructionInfo(64, "C7 44 48 12 88A9CBED", Code.Mov_rm32_imm32, a => a.SpaceBetweenMemoryAddOperators = true),
			new OptionsInstructionInfo(64, "C7 44 48 12 88A9CBED", Code.Mov_rm32_imm32, a => a.SpaceBetweenMemoryAddOperators = false),

			new OptionsInstructionInfo(64, "C7 44 48 EE 88A9CBED", Code.Mov_rm32_imm32, a => { a.ScaleBeforeIndex = false; a.SpaceBetweenMemoryMulOperators = true; }),
			new OptionsInstructionInfo(64, "C7 44 48 EE 88A9CBED", Code.Mov_rm32_imm32, a => { a.ScaleBeforeIndex = false; a.SpaceBetweenMemoryMulOperators = false; }),

			new OptionsInstructionInfo(64, "C7 44 48 EE 88A9CBED", Code.Mov_rm32_imm32, a => { a.ScaleBeforeIndex = true; a.SpaceBetweenMemoryMulOperators = true; }),
			new OptionsInstructionInfo(64, "C7 44 48 EE 88A9CBED", Code.Mov_rm32_imm32, a => { a.ScaleBeforeIndex = true; a.SpaceBetweenMemoryMulOperators = false; }),

			new OptionsInstructionInfo(64, "8B 0C 10", Code.Mov_r32_rm32, a => a.AlwaysShowScale = true),
			new OptionsInstructionInfo(64, "8B 0C 10", Code.Mov_r32_rm32, a => a.AlwaysShowScale = false),

			new OptionsInstructionInfo(64, "8B 0C 10", Code.Mov_r32_rm32, a => a.AlwaysShowSegmentRegister = true),
			new OptionsInstructionInfo(64, "8B 0C 10", Code.Mov_r32_rm32, a => a.AlwaysShowSegmentRegister = false),

			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.HexPrefix = null; a.HexSuffix = null; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.HexPrefix = "ppp"; a.HexSuffix = null; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.HexPrefix = null; a.HexSuffix = "sss"; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.HexPrefix = ""; a.HexSuffix = ""; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.HexPrefix = "ppp"; a.HexSuffix = ""; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.HexPrefix = ""; a.HexSuffix = "sss"; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.HexPrefix = "ppp"; a.HexSuffix = "sss"; }),

			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DecimalPrefix = null; a.DecimalSuffix = null; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DecimalPrefix = "ppp"; a.DecimalSuffix = null; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DecimalPrefix = null; a.DecimalSuffix = "sss"; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DecimalPrefix = ""; a.DecimalSuffix = ""; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DecimalPrefix = "ppp"; a.DecimalSuffix = ""; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DecimalPrefix = ""; a.DecimalSuffix = "sss"; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DecimalPrefix = "ppp"; a.DecimalSuffix = "sss"; }),

			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.OctalPrefix = null; a.OctalSuffix = null; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.OctalPrefix = "ppp"; a.OctalSuffix = null; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.OctalPrefix = null; a.OctalSuffix = "sss"; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.OctalPrefix = ""; a.OctalSuffix = ""; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.OctalPrefix = "ppp"; a.OctalSuffix = ""; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.OctalPrefix = ""; a.OctalSuffix = "sss"; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.OctalPrefix = "ppp"; a.OctalSuffix = "sss"; }),

			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Binary; a.BinaryPrefix = null; a.BinarySuffix = null; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Binary; a.BinaryPrefix = "ppp"; a.BinarySuffix = null; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Binary; a.BinaryPrefix = null; a.BinarySuffix = "sss"; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Binary; a.BinaryPrefix = ""; a.BinarySuffix = ""; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Binary; a.BinaryPrefix = "ppp"; a.BinarySuffix = ""; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Binary; a.BinaryPrefix = ""; a.BinarySuffix = "sss"; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Binary; a.BinaryPrefix = "ppp"; a.BinarySuffix = "sss"; }),

			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = null; a.HexDigitGroupSize = 0; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = null; a.HexDigitGroupSize = 4; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 1; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = null; a.HexDigitGroupSize = 1; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; }),

			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = null; a.DecimalDigitGroupSize = 0; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = null; a.DecimalDigitGroupSize = 3; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.DecimalDigitGroupSize = 0; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 1; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = null; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 1; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; }),

			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = null; a.OctalDigitGroupSize = 0; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = null; a.OctalDigitGroupSize = 4; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.OctalDigitGroupSize = 0; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 1; a.BinaryDigitGroupSize = 0; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = null; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 1; a.BinaryDigitGroupSize = 0; }),

			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = null; a.BinaryDigitGroupSize = 0; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = null; a.BinaryDigitGroupSize = 4; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.BinaryDigitGroupSize = 0; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 1; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADC7E", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = null; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 1; }),

			new OptionsInstructionInfo(64, "48 C7 C000000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C010000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000010000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000100000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000000100", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000001000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000000001", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000000010", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000001000000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000010000000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000010000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000100000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000000100", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000001000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000000001", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000000010", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),

			new OptionsInstructionInfo(64, "48 C7 C000000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C00A000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C064000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C0E8030000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C010270000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C0A0860100", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C040420F00", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C080969800", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000E1F505", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000CA9A3B", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 00E40B5402000000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 00E8764817000000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0010A5D4E8000000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 00A0724E18090000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 00407A10F35A0000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0080C6A47E8D0300", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000C16FF2862300", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 00008A5D78456301", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 000064A7B3B6E00D", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000E8890423C78A", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),

			new OptionsInstructionInfo(64, "48 C7 C000000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C008000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C040000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000020000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000100000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000800000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000000400", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000002000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000000001", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000000008", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000000040", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000002000000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000010000000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000080000000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000040000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000200000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000000100", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000000800", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000004000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000000002", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000000010", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B8 0000000000000080", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = false; }),

			new OptionsInstructionInfo(64, "48 C7 C000000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C002000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C004000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C008000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C010000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C020000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C040000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C080000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000010000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000020000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000040000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000080000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000100000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000200000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000400000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 C7 C000800000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = false; }),

			new OptionsInstructionInfo(64, "C7 C000000010", Code.Mov_rm32_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "C7 C080969800", Code.Mov_rm32_imm32, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "C7 C000002000", Code.Mov_rm32_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "C7 C080000000", Code.Mov_rm32_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = true; }),

			new OptionsInstructionInfo(64, "48 C7 C000000010", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 4; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "48 C7 C080969800", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Decimal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 3; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "48 C7 C000002000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Octal; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 4; a.BinaryDigitGroupSize = 0; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "48 C7 C080000000", Code.Mov_rm64_imm32, a => { a.NumberBase = NumberBase.Binary; a.DigitSeparator = "`"; a.HexDigitGroupSize = 0; a.DecimalDigitGroupSize = 0; a.OctalDigitGroupSize = 0; a.BinaryDigitGroupSize = 4; a.LeadingZeroes = true; }),

			new OptionsInstructionInfo(64, "48 B8 1032547698BADCFE", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.UpperCaseHex = true; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADCFE", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.UpperCaseHex = false; }),

			new OptionsInstructionInfo(64, "8A 40 F6", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 F7", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 F8", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 F9", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 FA", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 FB", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 FC", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 FD", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 FE", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 00", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 02", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 03", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 04", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 05", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 06", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 07", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 08", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 09", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),
			new OptionsInstructionInfo(64, "8A 40 0A", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = true; }),

			new OptionsInstructionInfo(64, "8A 40 F6", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 F7", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 F8", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 F9", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 FA", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 FB", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 FC", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 FD", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 FE", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 00", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 02", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 03", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 04", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 05", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 06", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 07", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 08", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 09", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),
			new OptionsInstructionInfo(64, "8A 40 0A", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.ShowZeroDisplacements = true; a.SmallHexNumbersInDecimal = false; }),

			new OptionsInstructionInfo(64, "48 B8 1032547698BADCFE", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.HexPrefix = null; a.HexSuffix = null; a.AddLeadingZeroToHexNumbers = true; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADCFE", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.HexPrefix = null; a.HexSuffix = null; a.AddLeadingZeroToHexNumbers = false; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADCFE", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.HexPrefix = ""; a.HexSuffix = ""; a.AddLeadingZeroToHexNumbers = true; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADCFE", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.HexPrefix = ""; a.HexSuffix = ""; a.AddLeadingZeroToHexNumbers = false; }),

			new OptionsInstructionInfo(64, "8A 40 01", Code.Mov_r8_rm8, a => a.MemorySizeOptions = MemorySizeOptions.Always),
			new OptionsInstructionInfo(64, "8A 40 01", Code.Mov_r8_rm8, a => a.MemorySizeOptions = MemorySizeOptions.Default),

			new OptionsInstructionInfo(64, "8A 0D 78563412", Code.Mov_r8_rm8, a => a.RipRelativeAddresses = false),
			new OptionsInstructionInfo(64, "67 8A 0D 78563412", Code.Mov_r8_rm8, a => a.RipRelativeAddresses = false),
			new OptionsInstructionInfo(64, "8A 0D 78563412", Code.Mov_r8_rm8, a => a.RipRelativeAddresses = true),
			new OptionsInstructionInfo(64, "67 8A 0D 78563412", Code.Mov_r8_rm8, a => a.RipRelativeAddresses = true),

			new OptionsInstructionInfo(64, "70 00", Code.Jo_rel8_64, a => a.ShowBranchSize = true),
			new OptionsInstructionInfo(64, "70 00", Code.Jo_rel8_64, a => a.ShowBranchSize = false),

			new OptionsInstructionInfo(64, "48 B8 1032547698BADCFE", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.UpperCaseHex = false; a.HexPrefix = "ppp"; a.HexSuffix = "sss"; a.UpperCaseAll = true; }),
			new OptionsInstructionInfo(64, "48 B8 1032547698BADCFE", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.UpperCaseHex = false; a.HexPrefix = "ppp"; a.HexSuffix = "sss"; a.UpperCaseAll = false; }),

			new OptionsInstructionInfo(64, "70 00", Code.Jo_rel8_64, a => { a.SmallHexNumbersInDecimal = false; a.BranchLeadingZeroes = false; }, a => a.IP = 0),
			new OptionsInstructionInfo(64, "70 00", Code.Jo_rel8_64, a => { a.SmallHexNumbersInDecimal = false; a.BranchLeadingZeroes = true; }, a => a.IP = 0),

			new OptionsInstructionInfo(64, "B1 A5", Code.Mov_r8_imm8, a => a.SignedImmediateOperands = true),
			new OptionsInstructionInfo(64, "B1 A5", Code.Mov_r8_imm8, a => a.SignedImmediateOperands = false),
			new OptionsInstructionInfo(64, "66 B9 5AA5", Code.Mov_r16_imm16, a => a.SignedImmediateOperands = true),
			new OptionsInstructionInfo(64, "66 B9 5AA5", Code.Mov_r16_imm16, a => a.SignedImmediateOperands = false),
			new OptionsInstructionInfo(64, "B9 34125AA5", Code.Mov_r32_imm32, a => a.SignedImmediateOperands = true),
			new OptionsInstructionInfo(64, "B9 34125AA5", Code.Mov_r32_imm32, a => a.SignedImmediateOperands = false),
			new OptionsInstructionInfo(64, "48 B9 7856341234125AA5", Code.Mov_r64_imm64, a => a.SignedImmediateOperands = true),
			new OptionsInstructionInfo(64, "48 B9 7856341234125AA5", Code.Mov_r64_imm64, a => a.SignedImmediateOperands = false),
			new OptionsInstructionInfo(64, "66 83 C9 A5", Code.Or_rm16_imm8, a => a.SignedImmediateOperands = true),
			new OptionsInstructionInfo(64, "66 83 C9 A5", Code.Or_rm16_imm8, a => a.SignedImmediateOperands = false),
			new OptionsInstructionInfo(64, "83 C9 A5", Code.Or_rm32_imm8, a => a.SignedImmediateOperands = true),
			new OptionsInstructionInfo(64, "83 C9 A5", Code.Or_rm32_imm8, a => a.SignedImmediateOperands = false),
			new OptionsInstructionInfo(64, "48 83 C9 A5", Code.Or_rm64_imm8, a => a.SignedImmediateOperands = true),
			new OptionsInstructionInfo(64, "48 83 C9 A5", Code.Or_rm64_imm8, a => a.SignedImmediateOperands = false),
			new OptionsInstructionInfo(64, "48 81 C9 34125AA5", Code.Or_rm64_imm32, a => a.SignedImmediateOperands = true),
			new OptionsInstructionInfo(64, "48 81 C9 34125AA5", Code.Or_rm64_imm32, a => a.SignedImmediateOperands = false),

			new OptionsInstructionInfo(16, "8A 40 00", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = true),
			new OptionsInstructionInfo(16, "8A 40 00", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = false),
			new OptionsInstructionInfo(16, "8A 80 0000", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = true),
			new OptionsInstructionInfo(16, "8A 80 0000", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = false),
			new OptionsInstructionInfo(16, "8A 06 0000", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = true),
			new OptionsInstructionInfo(16, "8A 06 0000", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = false),
			new OptionsInstructionInfo(32, "8A 05 00000000", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = true),
			new OptionsInstructionInfo(32, "8A 05 00000000", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = false),
			new OptionsInstructionInfo(32, "8A 40 00", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = true),
			new OptionsInstructionInfo(32, "8A 40 00", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = false),
			new OptionsInstructionInfo(32, "8A 80 00000000", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = true),
			new OptionsInstructionInfo(32, "8A 80 00000000", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = false),
			new OptionsInstructionInfo(32, "8A 04 25 00000000", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = true),
			new OptionsInstructionInfo(32, "8A 04 25 00000000", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = false),
			new OptionsInstructionInfo(32, "8A 44 40 00", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = true),
			new OptionsInstructionInfo(32, "8A 44 40 00", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = false),
			new OptionsInstructionInfo(32, "8A 84 40 00000000", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = true),
			new OptionsInstructionInfo(32, "8A 84 40 00000000", Code.Mov_r8_rm8, a => a.ShowZeroDisplacements = false),
			new OptionsInstructionInfo(64, "8A 05 00000000", Code.Mov_r8_rm8, a => { a.ShowZeroDisplacements = true; a.RipRelativeAddresses = true; }, a => a.IP = ulong.MaxValue - 6 + 1),
			new OptionsInstructionInfo(64, "8A 05 00000000", Code.Mov_r8_rm8, a => { a.ShowZeroDisplacements = false; a.RipRelativeAddresses = true; }, a => a.IP = ulong.MaxValue - 6 + 1),
			new OptionsInstructionInfo(64, "8A 05 00000000", Code.Mov_r8_rm8, a => { a.ShowZeroDisplacements = true; a.RipRelativeAddresses = false; }, a => a.IP = ulong.MaxValue - 6 + 1),
			new OptionsInstructionInfo(64, "8A 05 00000000", Code.Mov_r8_rm8, a => { a.ShowZeroDisplacements = false; a.RipRelativeAddresses = false; }, a => a.IP = ulong.MaxValue - 6 + 1),

			new OptionsInstructionInfo(64, "B1 00", Code.Mov_r8_imm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "B1 00", Code.Mov_r8_imm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "66 B9 0000", Code.Mov_r16_imm16, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "66 B9 0000", Code.Mov_r16_imm16, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "B9 00000000", Code.Mov_r32_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "B9 00000000", Code.Mov_r32_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "48 B9 0000000000000000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 B9 0000000000000000", Code.Mov_r64_imm64, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "66 83 C9 00", Code.Or_rm16_imm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "66 83 C9 00", Code.Or_rm16_imm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "83 C9 00", Code.Or_rm32_imm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "83 C9 00", Code.Or_rm32_imm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "48 83 C9 00", Code.Or_rm64_imm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 83 C9 00", Code.Or_rm64_imm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "48 81 C9 00000000", Code.Or_rm64_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "48 81 C9 00000000", Code.Or_rm64_imm32, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 06 0000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(16, "8A 06 0000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 05 00000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(32, "8A 05 00000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 04 25 00000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "8A 04 25 00000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.LeadingZeroes = true; }),

			new OptionsInstructionInfo(16, "8A 4F FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(16, "8A 4F FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 4F 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(16, "8A 4F 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 8F FFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(16, "8A 8F FFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 8F 0100", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(16, "8A 8F 0100", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(32, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(32, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 88 FFFFFFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(32, "8A 88 FFFFFFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 88 01000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(32, "8A 88 01000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 88 FFFFFFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "8A 88 FFFFFFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 88 01000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "8A 88 01000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),

			new OptionsInstructionInfo(16, "8A 4F FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(16, "8A 4F FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 4F 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(16, "8A 4F 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 8F FFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(16, "8A 8F FFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 8F 0100", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(16, "8A 8F 0100", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(32, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(32, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 88 FFFFFFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(32, "8A 88 FFFFFFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 88 01000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(32, "8A 88 01000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 88 FFFFFFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "8A 88 FFFFFFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 88 01000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "8A 88 01000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),

			new OptionsInstructionInfo(16, "8A 06 FFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(16, "8A 06 FFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(32, "8A 05 FFFFFFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(32, "8A 05 FFFFFFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "8A 04 25 FFFFFFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "8A 04 25 FFFFFFFF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),

			new OptionsInstructionInfo(16, "8A 06 0000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(16, "8A 06 0000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 05 00000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(32, "8A 05 00000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 04 25 00000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = false; }),
			new OptionsInstructionInfo(64, "8A 04 25 00000000", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),

			new OptionsInstructionInfo(16, "8A 4F FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = true; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 4F FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = true; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = true; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 4F 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = true; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 4F 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = true; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = true; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false; a.LeadingZeroes = true; }),

			new OptionsInstructionInfo(16, "8A 4F FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = true; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 4F FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = true; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = true; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 48 FF", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 4F 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = true; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "8A 4F 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = true; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = true; a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "8A 48 01", Code.Mov_r8_rm8, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false; a.LeadingZeroes = true; }),

			new OptionsInstructionInfo(16, "62 F24D4B BC 50 7F", Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "62 F24D4B BC 50 80", Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "62 F24D4B BC 50 7F", Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(16, "62 F24D4B BC 50 80", Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "62 F24D4B BC 50 7F", Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "62 F24D4B BC 50 80", Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "62 F24D4B BC 50 7F", Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(32, "62 F24D4B BC 50 80", Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "62 F24D4B BC 50 7F", Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "62 F24D4B BC 50 80", Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = true; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "62 F24D4B BC 50 7F", Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),
			new OptionsInstructionInfo(64, "62 F24D4B BC 50 80", Code.EVEX_Vfnmadd231ps_zmm_k1z_zmm_zmmm512b32_er, a => { a.NumberBase = NumberBase.Hexadecimal; a.SmallHexNumbersInDecimal = false; a.SignedMemoryDisplacements = false; a.DisplacementLeadingZeroes = false;  a.LeadingZeroes = true; }),

			new OptionsInstructionInfo(64, "DE C1", Code.Faddp_sti_st0, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "D8 D1", Code.Fcom_st0_sti, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "D8 D9", Code.Fcomp_st0_sti, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "DE F9", Code.Fdivp_sti_st0, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "DE F1", Code.Fdivrp_sti_st0, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "DE C9", Code.Fmulp_sti_st0, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "DE E9", Code.Fsubp_sti_st0, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "DE E1", Code.Fsubrp_sti_st0, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "DD E1", Code.Fucom_st0_sti, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "DD E9", Code.Fucomp_st0_sti, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "D9 C9", Code.Fxch_st0_sti, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "0FC2 CD 00", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "0FC2 CD 01", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "0FC2 CD 02", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "0FC2 CD 03", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "0FC2 CD 04", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "0FC2 CD 05", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "0FC2 CD 06", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "0FC2 CD 07", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "0FC2 CD 08", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "C5C8 C2 D3 00", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 01", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 02", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 03", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 04", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 05", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 06", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 07", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 08", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 09", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 0A", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 0B", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 0C", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 0D", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 0E", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 0F", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 10", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 11", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 12", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 13", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 14", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 15", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 16", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 17", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 18", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 19", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 1A", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 1B", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 1C", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 1D", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 1E", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 1F", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 20", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "C5CC C2 D3 00", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 01", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 02", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 03", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 04", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 05", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 06", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 07", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 08", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 09", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 0A", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 0B", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 0C", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 0D", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 0E", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 0F", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 10", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 11", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 12", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 13", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 14", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 15", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 16", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 17", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 18", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 19", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 1A", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 1B", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 1C", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 1D", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 1E", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 1F", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CC C2 D3 20", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 00", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 01", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 02", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 03", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 04", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 05", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 06", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 07", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 08", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 09", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 0A", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 0B", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 0C", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 0D", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 0E", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 0F", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 10", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 11", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 12", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 13", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 14", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 15", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 16", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 17", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 18", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 19", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 1A", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 1B", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 1C", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 1D", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 1E", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 1F", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 20", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 00", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 01", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 02", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 03", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 04", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 05", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 06", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 07", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 08", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 09", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 0A", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 0B", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 0C", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 0D", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 0E", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 0F", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 10", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 11", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 12", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 13", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 14", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 15", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 16", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 17", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 18", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 19", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 1A", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 1B", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 1C", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 1D", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 1E", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 1F", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 20", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 00", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 01", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 02", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 03", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 04", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 05", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 06", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 07", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 08", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 09", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 0A", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 0B", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 0C", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 0D", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 0E", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 0F", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 10", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 11", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 12", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 13", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 14", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 15", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 16", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 17", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 18", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 19", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 1A", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 1B", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 1C", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 1D", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 1E", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 1F", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 20", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "66 0FC2 CD 00", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 0FC2 CD 01", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 0FC2 CD 02", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 0FC2 CD 03", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 0FC2 CD 04", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 0FC2 CD 05", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 0FC2 CD 06", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 0FC2 CD 07", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 0FC2 CD 08", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "C5C9 C2 D3 00", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 01", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 02", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 03", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 04", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 05", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 06", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 07", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 08", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 09", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 0A", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 0B", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 0C", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 0D", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 0E", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 0F", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 10", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 11", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 12", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 13", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 14", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 15", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 16", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 17", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 18", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 19", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 1A", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 1B", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 1C", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 1D", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 1E", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 1F", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 20", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "C5CD C2 D3 00", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 01", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 02", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 03", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 04", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 05", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 06", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 07", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 08", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 09", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 0A", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 0B", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 0C", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 0D", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 0E", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 0F", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 10", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 11", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 12", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 13", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 14", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 15", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 16", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 17", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 18", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 19", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 1A", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 1B", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 1C", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 1D", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 1E", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 1F", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CD C2 D3 20", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 00", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 01", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 02", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 03", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 04", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 05", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 06", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 07", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 08", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 09", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 0A", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 0B", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 0C", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 0D", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 0E", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 0F", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 10", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 11", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 12", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 13", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 14", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 15", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 16", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 17", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 18", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 19", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 1A", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 1B", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 1C", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 1D", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 1E", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 1F", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 20", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 00", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 01", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 02", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 03", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 04", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 05", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 06", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 07", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 08", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 09", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 0A", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 0B", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 0C", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 0D", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 0E", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 0F", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 10", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 11", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 12", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 13", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 14", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 15", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 16", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 17", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 18", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 19", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 1A", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 1B", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 1C", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 1D", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 1E", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 1F", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 20", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 00", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 01", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 02", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 03", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 04", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 05", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 06", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 07", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 08", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 09", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 0A", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 0B", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 0C", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 0D", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 0E", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 0F", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 10", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 11", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 12", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 13", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 14", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 15", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 16", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 17", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 18", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 19", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 1A", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 1B", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 1C", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 1D", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 1E", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 1F", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 20", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "F3 0FC2 CD 00", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 01", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 02", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 03", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 04", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 05", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 06", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 07", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 08", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "C5CA C2 D3 00", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 01", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 02", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 03", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 04", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 05", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 06", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 07", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 08", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 09", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 0A", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 0B", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 0C", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 0D", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 0E", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 0F", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 10", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 11", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 12", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 13", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 14", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 15", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 16", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 17", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 18", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 19", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 1A", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 1B", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 1C", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 1D", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 1E", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 1F", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CA C2 D3 20", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 00", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 01", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 02", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 03", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 04", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 05", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 06", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 07", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 08", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 09", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 0A", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 0B", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 0C", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 0D", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 0E", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 0F", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 10", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 11", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 12", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 13", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 14", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 15", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 16", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 17", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 18", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 19", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 1A", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 1B", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 1C", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 1D", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 1E", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 1F", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 20", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "F2 0FC2 CD 00", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 01", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 02", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 03", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 04", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 05", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 06", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 07", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 08", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "C5CB C2 D3 00", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 01", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 02", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 03", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 04", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 05", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 06", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 07", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 08", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 09", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 0A", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 0B", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 0C", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 0D", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 0E", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 0F", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 10", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 11", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 12", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 13", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 14", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 15", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 16", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 17", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 18", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 19", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 1A", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 1B", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 1C", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 1D", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 1E", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 1F", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C5CB C2 D3 20", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 00", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 01", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 02", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 03", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 04", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 05", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 06", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 07", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 08", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 09", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 0A", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 0B", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 0C", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 0D", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 0E", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 0F", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 10", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 11", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 12", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 13", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 14", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 15", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 16", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 17", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 18", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 19", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 1A", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 1B", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 1C", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 1D", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 1E", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 1F", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 20", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "8FE848 CC D3 00", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CC D3 01", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CC D3 02", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CC D3 03", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CC D3 04", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CC D3 05", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CC D3 06", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CC D3 07", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CC D3 08", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "8FE848 CD D3 00", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CD D3 01", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CD D3 02", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CD D3 03", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CD D3 04", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CD D3 05", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CD D3 06", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CD D3 07", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CD D3 08", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "8FE848 CE D3 00", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CE D3 01", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CE D3 02", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CE D3 03", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CE D3 04", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CE D3 05", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CE D3 06", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CE D3 07", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CE D3 08", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "8FE848 CF D3 00", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CF D3 01", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CF D3 02", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CF D3 03", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CF D3 04", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CF D3 05", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CF D3 06", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CF D3 07", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 CF D3 08", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "8FE848 EC D3 00", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EC D3 01", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EC D3 02", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EC D3 03", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EC D3 04", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EC D3 05", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EC D3 06", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EC D3 07", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EC D3 08", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "8FE848 ED D3 00", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 ED D3 01", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 ED D3 02", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 ED D3 03", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 ED D3 04", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 ED D3 05", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 ED D3 06", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 ED D3 07", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 ED D3 08", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "8FE848 EE D3 00", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EE D3 01", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EE D3 02", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EE D3 03", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EE D3 04", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EE D3 05", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EE D3 06", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EE D3 07", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EE D3 08", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "8FE848 EF D3 00", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EF D3 01", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EF D3 02", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EF D3 03", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EF D3 04", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EF D3 05", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EF D3 06", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EF D3 07", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "8FE848 EF D3 08", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "DE C1", Code.Faddp_sti_st0, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "D8 D1", Code.Fcom_st0_sti, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "D8 D9", Code.Fcomp_st0_sti, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "DE F9", Code.Fdivp_sti_st0, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "DE F1", Code.Fdivrp_sti_st0, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "DE C9", Code.Fmulp_sti_st0, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "DE E9", Code.Fsubp_sti_st0, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "DE E1", Code.Fsubrp_sti_st0, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "DD E1", Code.Fucom_st0_sti, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "DD E9", Code.Fucomp_st0_sti, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "D9 C9", Code.Fxch_st0_sti, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "0FC2 CD 00", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "0FC2 CD 01", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "0FC2 CD 02", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "0FC2 CD 03", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "0FC2 CD 04", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "0FC2 CD 05", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "0FC2 CD 06", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "0FC2 CD 07", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "0FC2 CD 08", Code.Cmpps_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "C5C8 C2 D3 00", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 01", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 02", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 03", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 04", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 05", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 06", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 07", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 08", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 09", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 0A", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 0B", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 0C", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 0D", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 0E", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 0F", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 10", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 11", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 12", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 13", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 14", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 15", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 16", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 17", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 18", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 19", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 1A", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 1B", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 1C", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 1D", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 1E", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 1F", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C8 C2 D3 20", Code.VEX_Vcmpps_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "C5CC C2 D3 00", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 01", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 02", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 03", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 04", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 05", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 06", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 07", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 08", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 09", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 0A", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 0B", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 0C", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 0D", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 0E", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 0F", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 10", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 11", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 12", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 13", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 14", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 15", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 16", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 17", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 18", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 19", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 1A", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 1B", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 1C", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 1D", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 1E", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 1F", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CC C2 D3 20", Code.VEX_Vcmpps_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 00", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 01", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 02", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 03", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 04", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 05", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 06", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 07", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 08", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 09", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 0A", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 0B", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 0C", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 0D", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 0E", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 0F", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 10", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 11", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 12", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 13", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 14", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 15", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 16", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 17", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 18", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 19", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 1A", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 1B", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 1C", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 1D", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 1E", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 1F", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C0B C2 D3 20", Code.EVEX_Vcmpps_k_k1_xmm_xmmm128b32_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 00", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 01", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 02", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 03", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 04", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 05", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 06", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 07", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 08", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 09", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 0A", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 0B", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 0C", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 0D", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 0E", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 0F", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 10", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 11", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 12", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 13", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 14", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 15", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 16", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 17", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 18", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 19", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 1A", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 1B", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 1C", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 1D", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 1E", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 1F", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C2B C2 D3 20", Code.EVEX_Vcmpps_k_k1_ymm_ymmm256b32_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 00", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 01", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 02", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 03", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 04", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 05", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 06", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 07", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 08", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 09", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 0A", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 0B", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 0C", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 0D", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 0E", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 0F", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 10", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 11", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 12", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 13", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 14", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 15", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 16", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 17", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 18", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 19", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 1A", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 1B", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 1C", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 1D", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 1E", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 1F", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14C5B C2 D3 20", Code.EVEX_Vcmpps_k_k1_zmm_zmmm512b32_imm8_sae, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "66 0FC2 CD 00", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 0FC2 CD 01", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 0FC2 CD 02", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 0FC2 CD 03", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 0FC2 CD 04", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 0FC2 CD 05", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 0FC2 CD 06", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 0FC2 CD 07", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 0FC2 CD 08", Code.Cmppd_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "C5C9 C2 D3 00", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 01", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 02", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 03", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 04", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 05", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 06", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 07", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 08", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 09", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 0A", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 0B", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 0C", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 0D", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 0E", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 0F", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 10", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 11", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 12", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 13", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 14", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 15", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 16", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 17", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 18", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 19", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 1A", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 1B", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 1C", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 1D", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 1E", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 1F", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5C9 C2 D3 20", Code.VEX_Vcmppd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "C5CD C2 D3 00", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 01", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 02", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 03", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 04", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 05", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 06", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 07", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 08", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 09", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 0A", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 0B", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 0C", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 0D", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 0E", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 0F", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 10", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 11", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 12", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 13", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 14", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 15", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 16", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 17", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 18", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 19", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 1A", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 1B", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 1C", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 1D", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 1E", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 1F", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CD C2 D3 20", Code.VEX_Vcmppd_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 00", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 01", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 02", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 03", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 04", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 05", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 06", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 07", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 08", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 09", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 0A", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 0B", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 0C", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 0D", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 0E", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 0F", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 10", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 11", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 12", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 13", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 14", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 15", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 16", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 17", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 18", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 19", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 1A", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 1B", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 1C", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 1D", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 1E", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 1F", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD0B C2 D3 20", Code.EVEX_Vcmppd_k_k1_xmm_xmmm128b64_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 00", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 01", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 02", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 03", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 04", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 05", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 06", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 07", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 08", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 09", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 0A", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 0B", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 0C", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 0D", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 0E", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 0F", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 10", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 11", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 12", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 13", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 14", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 15", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 16", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 17", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 18", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 19", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 1A", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 1B", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 1C", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 1D", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 1E", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 1F", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD2B C2 D3 20", Code.EVEX_Vcmppd_k_k1_ymm_ymmm256b64_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 00", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 01", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 02", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 03", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 04", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 05", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 06", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 07", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 08", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 09", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 0A", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 0B", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 0C", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 0D", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 0E", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 0F", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 10", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 11", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 12", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 13", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 14", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 15", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 16", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 17", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 18", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 19", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 1A", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 1B", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 1C", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 1D", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 1E", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 1F", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CD5B C2 D3 20", Code.EVEX_Vcmppd_k_k1_zmm_zmmm512b64_imm8_sae, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "F3 0FC2 CD 00", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 01", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 02", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 03", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 04", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 05", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 06", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 07", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F3 0FC2 CD 08", Code.Cmpss_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "C5CA C2 D3 00", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 01", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 02", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 03", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 04", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 05", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 06", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 07", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 08", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 09", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 0A", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 0B", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 0C", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 0D", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 0E", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 0F", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 10", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 11", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 12", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 13", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 14", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 15", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 16", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 17", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 18", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 19", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 1A", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 1B", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 1C", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 1D", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 1E", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 1F", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CA C2 D3 20", Code.VEX_Vcmpss_xmm_xmm_xmmm32_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 00", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 01", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 02", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 03", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 04", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 05", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 06", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 07", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 08", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 09", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 0A", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 0B", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 0C", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 0D", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 0E", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 0F", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 10", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 11", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 12", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 13", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 14", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 15", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 16", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 17", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 18", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 19", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 1A", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 1B", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 1C", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 1D", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 1E", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 1F", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F14E0B C2 D3 20", Code.EVEX_Vcmpss_k_k1_xmm_xmmm32_imm8_sae, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "F2 0FC2 CD 00", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 01", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 02", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 03", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 04", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 05", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 06", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 07", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "F2 0FC2 CD 08", Code.Cmpsd_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "C5CB C2 D3 00", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 01", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 02", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 03", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 04", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 05", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 06", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 07", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 08", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 09", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 0A", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 0B", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 0C", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 0D", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 0E", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 0F", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 10", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 11", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 12", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 13", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 14", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 15", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 16", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 17", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 18", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 19", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 1A", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 1B", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 1C", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 1D", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 1E", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 1F", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C5CB C2 D3 20", Code.VEX_Vcmpsd_xmm_xmm_xmmm64_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 00", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 01", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 02", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 03", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 04", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 05", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 06", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 07", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 08", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 09", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 0A", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 0B", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 0C", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 0D", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 0E", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 0F", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 10", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 11", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 12", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 13", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 14", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 15", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 16", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 17", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 18", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 19", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 1A", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 1B", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 1C", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 1D", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 1E", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 1F", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F1CF0B C2 D3 20", Code.EVEX_Vcmpsd_k_k1_xmm_xmmm64_imm8_sae, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "8FE848 CC D3 00", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CC D3 01", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CC D3 02", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CC D3 03", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CC D3 04", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CC D3 05", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CC D3 06", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CC D3 07", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CC D3 08", Code.XOP_Vpcomb_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "8FE848 CD D3 00", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CD D3 01", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CD D3 02", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CD D3 03", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CD D3 04", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CD D3 05", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CD D3 06", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CD D3 07", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CD D3 08", Code.XOP_Vpcomw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "8FE848 CE D3 00", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CE D3 01", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CE D3 02", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CE D3 03", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CE D3 04", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CE D3 05", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CE D3 06", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CE D3 07", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CE D3 08", Code.XOP_Vpcomd_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "8FE848 CF D3 00", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CF D3 01", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CF D3 02", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CF D3 03", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CF D3 04", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CF D3 05", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CF D3 06", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CF D3 07", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 CF D3 08", Code.XOP_Vpcomq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "8FE848 EC D3 00", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EC D3 01", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EC D3 02", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EC D3 03", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EC D3 04", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EC D3 05", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EC D3 06", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EC D3 07", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EC D3 08", Code.XOP_Vpcomub_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "8FE848 ED D3 00", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 ED D3 01", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 ED D3 02", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 ED D3 03", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 ED D3 04", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 ED D3 05", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 ED D3 06", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 ED D3 07", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 ED D3 08", Code.XOP_Vpcomuw_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "8FE848 EE D3 00", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EE D3 01", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EE D3 02", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EE D3 03", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EE D3 04", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EE D3 05", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EE D3 06", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EE D3 07", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EE D3 08", Code.XOP_Vpcomud_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "8FE848 EF D3 00", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EF D3 01", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EF D3 02", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EF D3 03", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EF D3 04", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EF D3 05", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EF D3 06", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EF D3 07", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "8FE848 EF D3 08", Code.XOP_Vpcomuq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "66 6B C9 FF", Code.Imul_r16_rm16_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "6B C9 FF", Code.Imul_r32_rm32_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "48 6B C9 FF", Code.Imul_r64_rm64_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 69 C9 34 12", Code.Imul_r16_rm16_imm16, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "69 C9 78 56 34 12", Code.Imul_r32_rm32_imm32, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "48 69 C9 78 56 34 12", Code.Imul_r64_rm64_imm32, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "66 6B C9 FF", Code.Imul_r16_rm16_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "6B C9 FF", Code.Imul_r32_rm32_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "48 6B C9 FF", Code.Imul_r64_rm64_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 69 C9 34 12", Code.Imul_r16_rm16_imm16, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "69 C9 78 56 34 12", Code.Imul_r32_rm32_imm32, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "48 69 C9 78 56 34 12", Code.Imul_r64_rm64_imm32, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "66 0F3A44 F4 00", Code.Pclmulqdq_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 0F3A44 F4 01", Code.Pclmulqdq_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 0F3A44 F4 10", Code.Pclmulqdq_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 0F3A44 F4 11", Code.Pclmulqdq_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "66 0F3A44 F4 FF", Code.Pclmulqdq_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "66 0F3A44 F4 00", Code.Pclmulqdq_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 0F3A44 F4 01", Code.Pclmulqdq_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 0F3A44 F4 10", Code.Pclmulqdq_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 0F3A44 F4 11", Code.Pclmulqdq_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "66 0F3A44 F4 FF", Code.Pclmulqdq_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "C4E369 44 F4 00", Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C4E369 44 F4 01", Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C4E369 44 F4 10", Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C4E369 44 F4 11", Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C4E369 44 F4 FF", Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "C4E369 44 F4 00", Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C4E369 44 F4 01", Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C4E369 44 F4 10", Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C4E369 44 F4 11", Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C4E369 44 F4 FF", Code.VEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "C4E36D 44 F4 00", Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C4E36D 44 F4 01", Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C4E36D 44 F4 10", Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C4E36D 44 F4 11", Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "C4E36D 44 F4 FF", Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "C4E36D 44 F4 00", Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C4E36D 44 F4 01", Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C4E36D 44 F4 10", Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C4E36D 44 F4 11", Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "C4E36D 44 F4 FF", Code.VEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "62 F36D08 44 F4 00", Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F36D08 44 F4 01", Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F36D08 44 F4 10", Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F36D08 44 F4 11", Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F36D08 44 F4 FF", Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "62 F36D08 44 F4 00", Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F36D08 44 F4 01", Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F36D08 44 F4 10", Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F36D08 44 F4 11", Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F36D08 44 F4 FF", Code.EVEX_Vpclmulqdq_xmm_xmm_xmmm128_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "62 F36D28 44 F4 00", Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F36D28 44 F4 01", Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F36D28 44 F4 10", Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F36D28 44 F4 11", Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F36D28 44 F4 FF", Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "62 F36D28 44 F4 00", Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F36D28 44 F4 01", Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F36D28 44 F4 10", Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F36D28 44 F4 11", Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F36D28 44 F4 FF", Code.EVEX_Vpclmulqdq_ymm_ymm_ymmm256_imm8, a => a.UsePseudoOps = true),

			new OptionsInstructionInfo(64, "62 F36D48 44 F4 00", Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F36D48 44 F4 01", Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F36D48 44 F4 10", Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F36D48 44 F4 11", Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8, a => a.UsePseudoOps = false),
			new OptionsInstructionInfo(64, "62 F36D48 44 F4 FF", Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8, a => a.UsePseudoOps = false),

			new OptionsInstructionInfo(64, "62 F36D48 44 F4 00", Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F36D48 44 F4 01", Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F36D48 44 F4 10", Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F36D48 44 F4 11", Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8, a => a.UsePseudoOps = true),
			new OptionsInstructionInfo(64, "62 F36D48 44 F4 FF", Code.EVEX_Vpclmulqdq_zmm_zmm_zmmm512_imm8, a => a.UsePseudoOps = true),
		};
	}
}
#endif
